
/* TSR Program and Applications */

#include "stdio.h"
#include "stdlib.h"
#include "ctype.h"
#include "fcntl.h"
#include "string.h"
#include "bios.h"
#include "io.h"
#include "dos.h"
#include "conio.h"
#include "vstrack1.h"

#define CLOCK_ADJUST 1
#define STK_SIZE 0x3000



/* TSR function */
void initVIO(void);
void Vio_TSR(void);
void interrupt TSR_Key_Stk(void);
void interrupt (*old_key)();
void interrupt (*old_int60)();
void cursor_pos(void);
void interrupt dos_idle(void);
void interrupt (*old_int28)();
void interrupt (*old_int8)();
void interrupt Int8_New(void);
void goto_X_Y(short x, short y);


char busy = 0;
int Vstart = 0;

unsigned char stck[STK_SIZE];
unsigned int sp, ss;

typedef struct {
  short angle;          // head angle (+2048 (left) -- 0 (center) -- -2048 (right))
  short pitch;        // looking up/down (-110 -- +90)
} V_mouse_t;

V_mouse_t Vmouse;

int old_x, old_y;
int key;
char far *dos_active;
static coord sensitivity;
static TrackerData td;
static TrackerStatus ts;
long yaw;
main(void)
{
 union REGS r;
 struct SREGS s;

 /* see if already loaded */
 old_int60 = getvect(0x60); /* unused interrupt */
 if(!old_int60) setvect(0x60, Int8_New); /* set a flag */
 else {
   printf("TSR already loaded.");
	exit (0);
 }

 /* obtain the dos_active flag address */
 r.h.ah = 0x34;
 int86x(0x21, &r, &r, &s);
 dos_active = MK_FP(s.es, r.x.bx);

 /* get old ISR addresses */
 old_key = getvect(9); /* keystroke int */
 old_int28 = getvect(0x28); /* int 28 */
 old_int8 = getvect(8); /* timer interrupt */

 /* re-route interrupt vectors to new handlers */
 setvect(9, TSR_Key_Stk);
 setvect(0x28, dos_idle);
 setvect(8, Int8_New);

 printf("   F4: Deactivate\n");

 key = 0; 

 keep(0, 2000); /* terminate and stay resident */

 return 0;
}

/* This is the function that intercepts the keystroke
  interrupt (int 9). */
void interrupt TSR_Key_Stk(void)
{
 int far *t2 = (int far *) 1050; /* address of head pointer */
 char far *t = (char far *) 1050; /* address of head pointer */

 (*old_key)(); /* first, call old keystroke ISR */

 if(*t != *(t+2)) {/* if not empty */
   t += *t-30+5; /* advance to the character position */
   if(*t>=59 && *t<=62) {
     switch(*t) { /* see what is in scan code */
			case 62: /* F4: deactivate TSR */
			key = 1;
			break;
	  }
	  *(t2+1) = *t2; /* zero buffer */
	}
 }
}

/* New interrupt 8. */
void interrupt Int8_New(void)
{
 (*old_int8)(); /* call original int 8 */

 /* If DOS is idle, TSR is not already active,
	 and if a hot key has been hit, activate the TSR. */
 if(!*dos_active && !busy && !key)
 
Vio_TSR();
}

/* This function intercepts int 0x28 (the DOS-idle
  interrupt).
*/
void interrupt dos_idle(void)
{
 (*old_int28)(); /* call old int 0x28 ISR */
 if(!busy && !key) Vio_TSR();
}


static long ref=0;
static long set=0;

/* Pop up the TSR. */
void Vio_TSR(void)
{
 /* set up TSR stack */
 disable(); /* disable interrupts */
   ss = _SS; 
   sp = _SP;
   _SS = _DS;
   _SP = (unsigned) &stck[STK_SIZE-2];
 enable();  /* enable interrupts */
 if(!Vstart) {
		initVIO();
		Vstart = 1; }

 cursor_pos(); /* get current cursor position */

  ts = readTracker(&td,timerSecs(1)*CLOCK_ADJUST);

  if (ts != TS_OK) {
	 ts = resetTracker(&td,timerTSecs(1)*CLOCK_ADJUST);
  } // if
  requestTrackerData(&td);

#if 0
  if (set < 30) {
	 set++;
	 ref = td.euler.y;
  } // if

  yaw = (td.euler.y - ref)/(180L*(1L<<2));

  Vmouse.pitch = (short)(td.euler.x>>14);
  Vmouse.angle = (short)yaw;
#else
  if (!set) {
	 set = 1;
	 ref = td.euler.y;
  } // if

  yaw = (td.euler.y - ref);

#define MYAW  TOFIXED(180.)
#define MYAW2 (MYAW*2)

  if (yaw > MYAW) {
	 yaw =  MYAW2 - yaw;
  } else if (yaw < -MYAW) {
	 yaw = -MYAW2 - yaw;
  } // if

  yaw >>= 7;

  Vmouse.pitch = (short)(td.euler.x>>13); // 14
  Vmouse.angle = (short)yaw;
  ref = td.euler.y;
#endif

  goto_X_Y(Vmouse.pitch,Vmouse.angle);
  /*
  m.h.ah = 2;
  m.h.dh = Vmouse.pitch;
  m.h.dl = Vmouse.angle;
  m.h.bh = 0;
  int86(0x10, &m,&m); */

 /* restore old stack */
 disable();
	_SP = sp;
	_SS = ss;
 enable();

 if(key==1) { /* reset old ISR routines and free memory */
	closeTracker(&td);
	setvect(8, old_int8);
	setvect(9, old_key);
	setvect(0x28, old_int28);
	setvect(0x64, old_int64);
	freemem(_psp); /* free program block */
 } else key = 0;
}

/* Return16-bit scan code from the keyboard */
readkey(void)
{
 union REGS m;

 m.h.ah = 0;
 return int86(0x16, &m, &m);
}

/* Send the cursor to the specified X,Y position. */
void goto_X_Y(short x,short y)
{
 union REGS m;

 m.h.ah = 2; /* cursor addressing function */
 m.h.dl = x; /* column coordinate */
 m.h.dh = y; /* row coordinate */
 m.h.bh = 0; /* video page */
 int86(0x10, &m, &m);
}

/* Read and save cursor coordinates */
void cursor_pos(void)
{

 union REGS m, n;

 m.h.bh = 0;
 m.h.ah = 3;
 int86(16, &m, &n);
 
 old_x = n.h.dh;
 old_y = n.h.dl;
}

// Check Parm for com and sensitivty settings
int CheckParm(char * parm) {
int i;
  for(i = 1; i < _argc; i++) if(!strcmpi(_argv[i], parm)) return i;
  return 0; // parm not found
} // CheckParm

// initVIO used to initialize tracker

void initVIO(void) {
TrackerStatus ts;
long port=0,tport;
int i;

  if (!(i = CheckParm("-com"))) {
    TrackerPort trackerPort;
    printf("\nSearching for a Tracker: "); fflush(stdout);
    ts = initTrackerSearch(&td,&trackerPort,9600,timerSecs(2));
    printf("%s.\n",trackerInfo(ts));
    if (ts != TS_OK) {
      closeTracker(&td);
      exit(1);
    } // if
  } else {
	if (!sscanf(_argv[i+1],"%ld",&port)) {
      printf("A COM port (1-4) must be specified. Example: -com 1\n");
	  exit(1);
	} // if

    switch (port) {
      case 1: tport = TP_COM1; break;
      case 2: tport = TP_COM2; break;
      case 3: tport = TP_COM3; break;
      case 4: tport = TP_COM4; break;
      default:
	printf("Invalid COM port: %ld\n",port);
	exit(1);
    } // switch

    printf("Initializing Tracker on COM %ld.\n",port);

	 ts = initTracker(&td,tport,9600,timerSecs(2));
    printf("%s.\n",trackerInfo(ts));
    if (ts != TS_OK) {
      closeTracker(&td);
      exit(1);
    } // if

  } // if

  ts = sendTrackerCMD(&td,"!M1,P,B\r",timerSecs(2));
  if (ts != TS_OK) {
    printf("%s.\n",trackerInfo(ts));
    closeTracker(&td);
    exit(1);
  } // if

  requestTrackerData(&td);

} // InitVio

