//
// External 16-bit Real-mode Mouse Device Driver
//
//		Virtual I/O Tracker
//
// use with vstrack1.lib
// Date  : 12/27/95
// Name  : Ken Hawkins

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <dos.h>
#include <process.h>
#include <time.h>
#include <conio.h>
#include "vstrack1.h"

#define CLOCK_ADJUST 1

typedef unsigned char byte;

typedef struct {
  short vector;             // vector number to be called

  short angle;          // head angle (+2048 (left) -- 0 (center) -- -2048 (right))
  signed char pitch;        // looking up/down (-110 -- +90)
  unsigned short buttons;
} V_mouse_t;

V_mouse_t Vmouse;

static coord sensitivity;

short Vector;

void (interrupt *OldISR)();

//=========================================================================
//
//	CheckParm
//
//=========================================================================

int CheckParm(char * parm) {
int i;
  for(i = 1; i < _argc; i++) if(!strcmpi(_argv[i], parm)) return i;
  return 0; // parm not found
} // CheckParm

//=========================================================================
//
// InitVio
//
//		Initializes the VIO Tracker
//=========================================================================
static TrackerData td;

void InitVio(void) {
TrackerStatus ts;
long port=0,tport;
int i;

  if (!(i = CheckParm("-com"))) {
    TrackerPort trackerPort;
    printf("\nSearching for a Tracker: "); fflush(stdout);
    ts = initTrackerSearch(&td,&trackerPort,9600,timerSecs(2));
    printf("%s.\n",trackerInfo(ts));
    if (ts != TS_OK) {
      closeTracker(&td);
      exit(1);
    } // if
  } else {
	if (!sscanf(_argv[i+1],"%ld",&port)) {
      printf("A COM port (1-4) must be specified. Example: -com 1\n");
	  exit(1);
	} // if

    switch (port) {
      case 1: tport = TP_COM1; break;
      case 2: tport = TP_COM2; break;
      case 3: tport = TP_COM3; break;
      case 4: tport = TP_COM4; break;
      default:
        printf("Invalid COM port: %ld\n",port);
        exit(1);
    } // switch

    printf("Initializing Tracker on COM %ld.\n",port);

    ts = initTracker(&td,tport,9600,timerSecs(2));
    printf("%s.\n",trackerInfo(ts));
    if (ts != TS_OK) {
      closeTracker(&td);
      exit(1);
    } // if

  } // if

  ts = sendTrackerCMD(&td,"!M1,P,B\r",timerSecs(2));
  if (ts != TS_OK) {
    printf("%s.\n",trackerInfo(ts));
    closeTracker(&td);
    exit(1);
  } // if

  requestTrackerData(&td);

} // InitVio


//=========================================================================
//
// ExternISR
//
// 	Handles the low-level functionality of the External Device
//=========================================================================

static long ref=0;
static long set=0;

void interrupt V_mouse(void) {
TrackerStatus ts;
long yaw;

  ts = readTracker(&td,timerSecs(1)*CLOCK_ADJUST);

  if (ts != TS_OK) {
	 ts = resetTracker(&td,timerTSecs(1)*CLOCK_ADJUST);
  } // if
  requestTrackerData(&td);

#if 0
  if (set < 30) {
	 set++;
	 ref = td.euler.y;
  } // if

  yaw = (td.euler.y - ref)/(180L*(1L<<2));

  Vmouse.pitch = (short)(td.euler.x>>14);
  Vmouse.angle = (short)yaw;
#else
  if (!set) {
	 set = 1;
	 ref = td.euler.y;
  } // if

  yaw = (td.euler.y - ref);

#define MYAW  TOFIXED(180.)
#define MYAW2 (MYAW*2)

  if (yaw > MYAW) {
	 yaw =  MYAW2 - yaw;
  } else if (yaw < -MYAW) {
	 yaw = -MYAW2 - yaw;
  } // if

  yaw >>= 7;

  Vmouse.pitch = (short)(td.euler.x>>13); // 14
  Vmouse.angle = (short)yaw;
  ref = td.euler.y;
#endif
  regs.h.ah=2;
  regs.h.dh=Vmouse.pitch;
  regs.h.dl=Vmouse.angle;
  regs.h.bh=0;
  int86(0x10, &regs,&regs);
} // ExternISR

//=========================================================================
//
//	SetVector
//
//		Locates a vector (if one isn't passed with the '-vector' parameter)
//=========================================================================

void SetVector(void) {
unsigned char far *vect;
int i;

  if ((i = CheckParm("-vector")) != 0) {
	 sscanf(_argv[i+1],"0x%x",&Vmouse.vector);
  }	else {
	 for(Vmouse.vector = 0x60; Vmouse.vector < 0x67; Vmouse.vector++) {
		vect = *(char far * far *)(Vmouse.vector*4);
		if(!vect || *vect == 0xcf) break;
	 } // for
	 if (Vmouse.vector == 0x67) {
		printf("Unable to locate a free vector from (0x60-0x67).\nA vector may be specified with the '-vector' parameter.\n");
		closeTracker(&td);
		exit(1);
	 } // if
  } // if

  printf("Using interrupt vector 0x%x\n", Vmouse.vector);
  OldISR = _dos_getvect(Vmouse.vector);
  _dos_setvect(Vmouse.vector, V_mouse);
  Vector = Vmouse.vector;

} // SetVector

//=========================================================================
//
// Main
//
//		find a free a vector (if one isn't passed), and spawns HERETIC.EXE,
//			passing the address of the crossover structure.
//=========================================================================

void main(void) {
char *newargs[99];
long flataddr;
char addrstr[10];
int i;
union REGS regs;

  if(CheckParm("-?") || CheckParm("/?")) {
	 printf("\nHeretic VIO Tracker Driver.\nUsage: VIOHT [-sensi #] [-com #]\n");
	 printf("   -sensi #  : # is sensitivity in percent (10-400, default: 100).\n");
	 printf("   -com #    : # is the COM port number (1-4, default: autosearch).\n");
	 exit(1);
  } // if

  if ((i = CheckParm("-sensi")) != 0) {
	 if (!sscanf(_argv[i+1],"%ld",&sensitivity)) {
		printf("A number (10-400) must be specified after the -sensi argument.\n");
		printf("For example: -sensi 20, for 20%s sensitivity.\n","%");
		exit(1);
	 } // if
	 CLAMPMINMAX(sensitivity,10,400);
  } else {
	 sensitivity = 100;
  } // if

  sensitivity = DIV(sensitivity,100); // Convert % to fixed point

  memset(&Vmouse,0,sizeof(Vmouse));

  InitVio();
  SetVector();

  keep(0, 2000);
  return 0;

  /////////////////////////////////////////////////////////
  /* flataddr = (long)FP_SEG(&Vmouse)*16+(unsigned)FP_OFF(&Vmouse);

  sprintf(addrstr,"%lu",flataddr);
  memcpy(newargs,_argv,(_argc+1)*4);

  newargs[0]       = "heretic.exe";
  newargs[_argc]   = "-externdriver";
  newargs[_argc+1] = addrstr;
  newargs[_argc+2] = NULL;

  if (spawnv(P_WAIT,"heretic.exe",newargs) == -1) {
	 printf("\n  heretic.exe not found.\n");
  } // if

  _dos_setvect(Vector,OldExternISR);

  closeTracker(&td);

  printf("\nExited driver.\n"); */



} // main

