/* imath.h */
/* Created 12/14/94 */
/* Ken Hawkins */

#ifndef IMATH_H
#define IMATH_H

typedef long coord;
typedef unsigned short ushort;
typedef unsigned long ulong;
typedef struct {coord sin,cos;} sinCos;

#define FBITS 14L
#define SHR(n) ((n)>>FBITS)
#define SHL(n) ((n)<<FBITS)
#define ONE (1L<<FBITS)
#define FONE ((float)ONE)
#define TOFIXED(n) (long)((n)*ONE)
#define TOFLOAT(n) ((float)(n)/FONE)

#ifndef M_PI
#define M_PI 3.14159265358979323846264338327950288
#endif

#define M_TWOPI (M_PI*2)

#define RAD090DEG TOFIXED(M_PI/2.)
#define RAD180DEG TOFIXED(M_PI)
#define RAD270DEG TOFIXED(M_PI*3/2.)
#define RAD360DEG TOFIXED(M_PI*2.)

#define DEG090 TOFIXED(90.)
#define DEG180 TOFIXED(180.)
#define DEG270 TOFIXED(270.)
#define DEG360 TOFIXED(360.)

#define TORADIANS(a) MUL(a,TOFIXED(M_PI/180.))
#define TODEGREES(a) MUL(a,TOFIXED(180./M_PI))

#if defined(__WATCOMC__)

// The 14 below is FBITS
long mul64(long a,long b);
#pragma aux mul64 =      \
	"imul   edx"         \
	"shrd   eax,edx,14"  \
	"adc    eax,0"       \
	parm [eax] [edx]     \
	modify [eax edx]     \
	value [eax];

long div64(long a,long b);
#pragma aux div64 =      \
	"push   ebx"         \
	"mov    ebx,edx"     \
	"cdq"                \
	"shld   edx,eax,14"  \
	"shl    eax,14"      \
	"idiv   ebx"         \
	"pop    ebx"         \
	parm [eax] [edx]     \
	modify [eax edx]     \
	value [eax];

long muldiv64(long a,long b,long c);
#pragma aux muldiv64 =     \
	"imul    edx"          \
	"idiv    ebx"          \
	parm [eax] [edx] [ebx] \
	modify [eax edx ebx]   \
	value [eax];

#else

long mul64(long a,long b);
long div64(long a,long b);
long muldiv64(long a,long b,long c);

#endif

#define MUL(a,b) mul64(a,b)
#define DIV(a,b) div64(a,b)
#define MULDIV(a,b,c) muldiv64(a,b,c)

#ifdef abs
#undef abs
#endif
#define abs(x) ((x) < 0 ? -(x) : (x))

#define sgn(x) ((x) > 0 ? 1 : ((x) < 0 ? -1 : 0))

#define CLAMPMIN(n,min) if ((n) < (min)) (n) = (min);
#define CLAMPMAX(n,max) if ((n) > (max)) (n) = (max);
#define CLAMPMINMAX(n,min,max) CLAMPMIN(n,min) else CLAMPMAX(n,max)

ulong isqrt(ulong n);
coord getsin(coord radians);
coord getcos(coord radians);
coord getatan(coord sine);
coord getatan2(coord sine,coord cosine);
void sincos(sinCos * sc,coord radians);

#endif

/* imath.h */

/* vector.h */
/* Created 12/14/94 */
/* Ken Hawkins */

#ifndef VECTOR_H
#define VECTOR_H

typedef struct {coord x,y;} vec2;
typedef struct {coord x,y,z;} vec3;
typedef struct {coord x,y,z,w;} vec4;
typedef struct {vec3 x,y,z;} matrix33;
typedef struct {matrix33 m; vec3 w;} matrix34;
typedef struct {vec4 x,y,z,w;} matrix44;

typedef struct {vec3 d,t;} def3; // d)efinition, t)ransformed basis

#define SETVEC(v,a,b,c) (v).x=a;(v).y=b;(v).z=c
#define VZERO(a) (a).x=(a).y=(a).z=0
#define NEGVEC(a)    (a).x=-(a).x;(a).y=-(a).y;(a).z=-(a).z
#define NEGVECN(a,b) (a).x=-(b).x;(a).y=-(b).y;(a).z=-(b).z

#define ADDVEC(a,b,c) (a).x=(b).x+(c).x;(a).y=(b).y+(c).y;(a).z=(b).z+(c).z
#define INCVEC(a,b) (a).x+=(b).x;(a).y+=(b).y;(a).z+=(b).z;
#define SUBVEC(a,b,c) (a).x=(b).x-(c).x;(a).y=(b).y-(c).y;(a).z=(b).z-(c).z
#define DECVEC(a,b) (a).x-=(b).x;(a).y-=(b).y;(a).z-=(b).z
#define DOT2(a,b) (MUL((a).x,(b).x)+MUL((a).y,(b).y))
#define DOT3(a,b) (MUL((a).x,(b).x)+MUL((a).y,(b).y)+MUL((a).z,(b).z))
#define DOT4(a,b) (MUL((a).x,(b).x)+MUL((a).y,(b).y)+MUL((a).z,(b).z)+MUL((a).w,(b).w))

coord normalize2(vec2 * v);

coord normalize3(vec3 * v);

//*** Matrix/matrix, matrix/vector routines ***

void matrix33unitize(matrix33 * m);

void rotate33(vec3 * out,matrix33 * m,vec3 * in);

void rotate33rev(vec3 * out,matrix33 * m,vec3 * in);

void rotate34(vec3 * out,matrix34 * m,vec3 * in);

void rotate34rev(vec3 * out,matrix34 * m,vec3 * in);

void rotate34preTrans(vec3 * out,matrix34 * m,vec3 * in);

void matrixMul33(matrix33 * out,matrix33 * a,matrix33 * b);

void matrixMul34(matrix34 * out,matrix34 * a,matrix34 * b);

void rotate44(vec4 * out,matrix44 * m,vec4 * in);

void matrixMul44(matrix44 * out,matrix44 * a,matrix44 * b);

void matrix33axis(matrix33 * m,vec3 * axis,coord angle);

#endif

/* vector.h */
/* filter.h */
/* Created 11/22/94 */
/* Ken Hawkins */

#ifndef FILTER_H
#define FILTER_H

typedef struct filter {
  coord * history;
  coord * weights;
  long len;
  coord * hCurrent,* hEnd,* hRestart;
} filter;  

typedef struct ifilter {
  long filteredNum;
  long remainder;
  long filterRate;
} ifilter;

void initWeights(filter * f);
void initHistory(filter * f);
coord filterFIR(filter * f,coord newval);

long filterInterp(ifilter * f,long rawNum);

long initFIR(filter * f,long len);
void freeFIR(filter * f);

#endif

/* filter.h */
/* Convert.h */
/* Converts raw tracker data to orientation information */
/* Created 11/17/94 */
/* Ken Hawkins */

#ifndef VIO_CONVERT_H
#define VIO_CONVERT_H

void rawToEulerInt(long rawMagneticX,long rawMagneticY,long rawMagneticZ,
                   long rawGravimetricX,long rawGravimetricY,
                   long * eulerYaw,long * eulerPitch,long * eulerRoll);
                    
#endif

/* convert.h */
/* timer.h */
/* Created 12/12/94 */

#ifndef TIMER_H
#define TIMER_H

// These two functions are only needed if using the custom 18.2Hz timer.
// (see timer.c)
void initTimer(void);
void deInitTimer(void);


long getTime(void);

long elapsedTime(long starttime,long timeout);

long timerSecs(long secs);
long timerTSecs(long tsecs);

// Available if not using 18.2Hz timer (using system clock()):
long timerHSecs(long hsecs);

// 100th sec or 1/18.2 sec tics.
void delayTime(long tics);

float computeFrameRate(long frames,long starttime,long endtime);

#endif

/* timer.h */
/* serial.h
   Created 1/7/94, Modified 2/11/95
   Copyright 1995 Ken Hawkins
   All Rights Reserved
*/

#ifndef SERIAL_H
#define SERIAL_H

#define SERIAL_PORT_COM1 0
#define SERIAL_PORT_COM2 1
#define SERIAL_PORT_COM3 2
#define SERIAL_PORT_COM4 3
#define SERIAL_PORT_NONE 4

#define USE_16550 (1<<0)
#define SER_OK         0
#define SER_NO_PORT   -1
#define SER_NO_MEMORY -2

#define SER_NO_CHAR   -1

#define UART_NONE      0
#define UART_8250      1
#define UART_16550     2

#define FLG_CALLED (1<<4)

#ifdef __WATCOMC__
#pragma pack(1)
#define FAR
#else
#define FAR __far
#endif

typedef void __interrupt __far (__far *s_ifunc)(void);

typedef struct
{
  volatile short head;
  volatile short tail;
  volatile short count;
  short size;
  short seg;
} QUE;

typedef struct
{
  short jmp;
  short port;
  volatile short flags;
  QUE   rxq;
  QUE   txq;
} BIH_STRUCT;

typedef struct serQueue {
  QUE     FAR  *q;
  char    FAR  *que;
  short     selector;
} serQueue;

typedef struct serialData {
  serQueue tx,rx;
  long checkCTS;
  long fifoEnabled;
  long port;        /* Com1, com2, com3, or com4 */
  long portNum;     /* sdcom[4] index: 0..3 */
  long intvec;
  long intbit;
  BIH_STRUCT FAR *bih;
  unsigned short bihSel;
  unsigned short wRmSel;
  unsigned short wRmOfs;
  unsigned short wPmSel;
  unsigned long  wPmOfs;
  s_ifunc prevint;
  long prevLineControl;
  long prevModemControl;
  long prevPicMask;
  long prevIntEnable;
  long prevFifo;
  long prevBPS;
} serialData;

#ifdef __WATCOMC__
#pragma pack()
#endif

long initSerial(serialData * sd,long port,long flags,long txQsize,long rxQsize);
void deInitSerial(serialData * sd);

long setBPS(serialData * sd,long bpsRate);
long writeSer(serialData * sd,char ch);
void writeSerNoBuff(serialData * sd,char ch);
long readSer(serialData * sd);
long rxBuffEmpty(serialData * sd);
long txBuffEmpty(serialData * sd);
void flushRead(serialData * sd);
void flushWrite(serialData * sd);
void flushSer(serialData * sd);

void setCheckCTS(serialData * sd,long checkCTS);
void initDTR(serialData * sd);
void setDTR(serialData * sd,long state);
void setRTS(serialData * sd,long state);
long getDSR(serialData * sd);
long getDCD(serialData * sd);

long uartType(serialData * sd);

long serOpen(serialData * sd);

/* serial.h */

#endif
/* tracker.h */
/* Created 12/12/94 */
#ifndef TRACKER_H
#define TRACKER_H

#define TRACKER_VER_LEN 60

typedef enum {TP_COM1,TP_COM2,TP_COM3,TP_COM4} TrackerPort;
typedef enum {
  TS_OK,TS_NO_SERIAL,TS_NO_TRACKER,TS_TIMEOUT,
  TS_CHECKSUM,TS_HEADER,TS_WRITEVERIFY,TS_INVALID_PARM,
  TS_CMD_ERROR,TS_NO_MEMORY,TS_INVALID_VER_STR
} TrackerStatus;

typedef struct {
  void * sd;  // Serial Data
  vec3 magnetic;
  vec2 gravimetric;
  vec3 euler;
  vec3 magOff;
  vec3 magGain;
  vec2 grvOff;
  vec2 grvGain;
  long magFilts;
  long grvFilts;
} TrackerData;

extern long debugTrackerIO;

TrackerStatus initTracker(TrackerData * td,TrackerPort port,long comRate,long timeout);
TrackerStatus initTrackerSearch(TrackerData * td,TrackerPort * port,long comRate,long timeout);
void closeTracker(TrackerData * td);
TrackerStatus readPacket(TrackerData * td,char * data,long size,long timeout);
TrackerStatus readTracker(TrackerData * td,long timeout);
TrackerStatus readTrackerEuler(TrackerData * td,long timeout);
void writeSerString(TrackerData * td,char * s);
void requestTrackerData(TrackerData * td);
char * trackerInfo(TrackerStatus ts);

TrackerStatus resetTracker(TrackerData * td,long timeout);
TrackerStatus sendTrackerCMD(TrackerData * td,char * cmd,long timeout);
TrackerStatus getTrackerResponse(TrackerData * td,long timeout);

TrackerStatus getTrackerVerStr(TrackerData * td,char * str,long timeout);
TrackerStatus getTrackerVerRev(TrackerData * td,
                               long * hver,long * hrev,
                               long * fver,long * frev,
                               long timeout);

#endif

/* tracker.h */
/* trw.h */
/* Created 12/16/94 */
/* Ken Hawkins */

#ifndef TRW_H
#define TRW_H

TrackerStatus readTracker8(TrackerData * td,long addr,long * val);

TrackerStatus writeTracker8(TrackerData * td,long addr,long val);

TrackerStatus readTracker16(TrackerData * td,long addr,long * val);

TrackerStatus writeTracker16(TrackerData * td,long addr,long val);


#endif

/* trw.h */

/* tcal.h */
/* Created 12/16/94 */
/* Ken Hawkins */

#ifndef TCAL_H
#define TCAL_H

// Misc. var offsets               // Var size:
#define DATA_MODE_OFFSET      (0x04) // byte
#define MOUSE_MODE_OFFSET     (0x05) // byte
#define SEND_MODE_OFFSET      (0x06) // byte
#define DATA_FORMAT_OFFSET    (0x07) // byte

#define MOUSE_X_SENSI_OFFSET  (0x42) // byte
#define MOUSE_X_THRESH_OFFSET (0x43) // byte

#define MOUSE_Y_SENSI_OFFSET  (0x46) // byte
#define MOUSE_Y_THRESH_OFFSET (0x47) // byte

typedef enum {TA_X,TA_Y,TA_Z,TA_P,TA_R} TrackerAxis;

void setBaseAddr(unsigned long addr);

unsigned long getBaseAddr(void);

TrackerStatus readTrackerMiscVar(TrackerData * td,long offset,long * val);

TrackerStatus readTrackerGain(TrackerData * td,TrackerAxis ta);

TrackerStatus writeTrackerGain(TrackerData * td,TrackerAxis ta);

TrackerStatus readTrackerOffset(TrackerData * td,TrackerAxis ta);

TrackerStatus writeTrackerOffset(TrackerData * td,TrackerAxis ta);

TrackerStatus readTrackerOffsets(TrackerData * td);

TrackerStatus writeTrackerOffsets(TrackerData * td);

TrackerStatus readTrackerGains(TrackerData * td);

TrackerStatus writeTrackerGains(TrackerData * td);

TrackerStatus readTrackerMagFilts(TrackerData * td);

TrackerStatus writeTrackerMagFilts(TrackerData * td);

TrackerStatus readTrackerGrvFilts(TrackerData * td);

TrackerStatus writeTrackerGrvFilts(TrackerData * td);

TrackerStatus readTrackerFilters(TrackerData * td);

TrackerStatus writeTrackerFilters(TrackerData * td);

TrackerStatus readTrackerAllVars(TrackerData * td);

TrackerStatus writeTrackerAllVars(TrackerData * td);

char * getOffsetsStr(TrackerData * td);

char * getGainsStr(TrackerData * td);

char * getFiltersStr(TrackerData * td);

char * getAllVarsStr(TrackerData * td);

#endif

/* tcal.h */

/* spacerat.h */
/* Created 1/6/95 at CES in LAS VEGAS */
/* Ken Hawkins */

#ifndef SPACERAT_H
#define SPACERAT_H

void spacerat(TrackerData * td);

#endif

/* spacerat.h */

/* vtmode.h */
/* Created 3/21/95 */
/* Ken Hawkins */

#ifndef VTMODE_H
#define VTMODE_H

void vtmode(char * msg,int argc,char ** argv);

#endif

/* vtmode.h */

