/* tcal.c */
/* Created 12/30/94 */
/* Ken Hawkins */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include "imath1.h"
#include "vector1.h"
#include "tracker1.h"
#include "serial1.h"
#include "grid1.h"

#define VERSION "1.2"

#define LARROW 0x4b
#define RARROW 0x4d
#define UARROW 0x48
#define DARROW 0x50
#define F1 0x3b
#define F2 0x3c
#define F3 0x3d
#define F4 0x3e
#define F5 0x3f
#define F6 0x40
#define CR 0x0d
#define ESC 0x1b

#define CTO timerSecs(3)
#define RTO timerSecs(2)
#define DTO timerSecs(2)

TrackerData td;
static TrackerStatus ts;

void startTV(long smode);

// Returns -1 if abort, 0 if no input, 1 if input

long getInput(char * s) {
char * ts = s;
  while (1) {
	long ch = getch();
//printf("0x%x\n",ch);
	if (ch == ESC) return -1;
	if (ch == CR) {
	  if (ts == s) return 0;
	  *ts = 0;
	  return 1;
	} // if
	*ts++ = ch;
  } // while
} // getInput

long getDigitRange(long low,long hi,long * num) {
  while (1) {
	long ch;
	printf("(%d-%d):",low,hi); fflush(stdout);
	ch = getch();
	if (ch == ESC) return -1;
	if (ch == CR) return 0;
	ch -= '0';
	if (ch >= low && ch <= hi) {
	  *num = ch;
	  printf("%d\n",*num);
	  return 1;
	} // if
	printf("\n");
  } // while
} // getDigitRange

long getNumberRange(long low,long hi,long * num) {
char buff[256];
  while (1) {
	long status,tnum;
	printf("(%d-%d):",low,hi); fflush(stdout);
	status = getInput(buff);
	if (status < 1) return status;
	if (sscanf(buff,"%d",&tnum) == 1) {
	  if (tnum >= low && tnum <= hi) {
	*num = tnum;
	return 1;
	  } // if
	} // if
  } // while
} // getNumberRange

void displayCookedData(void) {
  printf("x %6d y %6d z %6d p %6d r %6d   \n",
	td.magnetic.x,td.magnetic.y,td.magnetic.z,
	td.gravimetric.x,td.gravimetric.y);
} // displayCookedData

void displayEulerData(void) {
  printf("Yaw %6.2f Pitch %6.2f Roll %6.2f  \n",
	TOFLOAT(td.euler.y),TOFLOAT(td.euler.x),TOFLOAT(td.euler.z));
} // displayEulerData

void doRequestData(void) {
TrackerStatus ts;
long status,mode;
  while (1) {
	printf("\n\nRequest Tracker Data:\n");
	printf("  1 = Cooked\n  2 = Euler\n  3 = microEuler\n");
	status = getDigitRange(1,3,&mode);
	if (status == 1) {
	  if (mode == 3) {

	flushSer(td.sd);
	ts = sendTrackerCMD(&td,"!M2,P,B\r",CTO);
	if (ts != TS_OK) {
	  printf("%s\n",trackerInfo(ts));
	  return;
	} // if

	requestTrackerData(&td);
	while (1) {
	  ts = readTrackerEuler(&td,DTO);
	  if (ts != TS_OK) {
		printf("%s\n",trackerInfo(ts));
		ts = resetTracker(&td,RTO);
		printf("%s\n",trackerInfo(ts));
	  } else {
		displayEulerData();
	  } // if

	  requestTrackerData(&td);

	  if (kbhit() && getch() == ESC) break;

	} // while

	  } else {

	flushSer(td.sd);

	ts = sendTrackerCMD(&td,"!M1,P,B\r",CTO);
	if (ts != TS_OK) {
	  printf("%s\n",trackerInfo(ts));
	  return;
	} // if

//      flushSer(td.sd);

	requestTrackerData(&td);
	while (1) {
	  ts = readTracker(&td,DTO);
	  if (ts != TS_OK) {
		printf("%s\n",trackerInfo(ts));
		ts = resetTracker(&td,RTO);
		printf("%s\n",trackerInfo(ts));
	  } else {
		switch (mode) {
		  case 1: displayCookedData(); break;
		  case 2: displayEulerData();  break;
		} // switch
	  } // if
	  requestTrackerData(&td);
	  if (kbhit() && getch() == ESC) break;
	} // while
	  } // if
	} else {
	  return;
	} // if
  } // while
} // doRequestData

void doMagneticCal(void) {
TrackerStatus ts;
long ch;
  printf("\nTracker Magnetic Calibration:\n");
  printf("Once tracker calibration has begun, it must be fully completed\n");
  printf("and cannot be undone. Press 'y' to continue:"); fflush(stdout);

  ch = getch();
  if (toupper(ch) != 'Y') return;

  printf("\nPress [RETURN] to start and [RETURN] when completed:"); fflush(stdout);

  while (1) {
	ch = getch();
	if (ch == ESC) return;
	if (ch == CR) break;
  } // while

//  while (!txBuffEmpty(td.sd)) if (kbhit() && getch() == ESC) return;

  flushSer(td.sd);

//  writeSer(td.sd,'c'); // Start calibration

  ts = sendTrackerCMD(&td,"!MC,M,B\r",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  writeSerString(&td,"S");

  while (1) {
	static unsigned char buff[256];
	TrackerStatus ts;
	short x,xmin,xmax,y,ymin,ymax,z,zmin,zmax;

	while (1) {
	  if (kbhit() && getch() == CR) break;

	  ts = readPacket(&td,buff,1,timerSecs(1));
	  if (ts != TS_OK) {
		printf("%s\n",trackerInfo(ts));
		goto done;
	  } // if

	  if (buff[0] == 255) break;

	} // while

	ts = readPacket(&td,buff,19,timerSecs(1)); // +1 = CRC added
	if (ts != TS_OK) {
	  printf("%s\n",trackerInfo(ts));
	  break;
	} // if

#define T_OFFSET 2048

	x    = (((short)buff[0]<<8)  | buff[1])  - T_OFFSET;
	xmax = (((short)buff[2]<<8)  | buff[3])  - T_OFFSET;
	xmin = (((short)buff[4]<<8)  | buff[5])  - T_OFFSET;
	y    = (((short)buff[6]<<8)  | buff[7])  - T_OFFSET;
	ymax = (((short)buff[8]<<8)  | buff[9])  - T_OFFSET;
	ymin = (((short)buff[10]<<8) | buff[11]) - T_OFFSET;
	z    = (((short)buff[12]<<8) | buff[13]) - T_OFFSET;
	zmax = (((short)buff[14]<<8) | buff[15]) - T_OFFSET;
	zmin = (((short)buff[16]<<8) | buff[17]) - T_OFFSET;

	printf("X: %6d %6d %6d Y: %6d %6d %6d Z: %6d %6d %6d\n",
	  x,xmin,xmax,y,ymin,ymax,z,zmin,zmax);

	if (kbhit() && getch() == CR) break;

  } // while

done:

  writeSerString(&td,"!");
  delayTime(timerHSecs(1));
  flushSer(td.sd);

  ts = sendTrackerCMD(&td,"R\r",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	ts = resetTracker(&td,RTO);
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  printf("Tracker Magnetic Calibration completed.\n");

} // doMagneticCal

void doGravimetricCal(void) {
TrackerStatus ts;
long ch;

  printf("\nTracker Gravimetric Calibration:\n");
  printf("Once tracker calibration has begun, it must be fully completed\n");
  printf("and cannot be undone. Press 'y' to continue:"); fflush(stdout);
  ch = getch();
  if (toupper(ch) != 'Y') return;

  flushSer(td.sd);

  ts = sendTrackerCMD(&td,"!MC,G,B\r",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  printf("\nPitch Tracker +20 degrees then press [RETURN]:"); fflush(stdout);
  while (getch() != CR);

  ts = sendTrackerCMD(&td,"S",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  printf("\nPitch Tracker -20 degrees then press [RETURN]:"); fflush(stdout);
  while (getch() != CR);

  ts = sendTrackerCMD(&td,"S",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  printf("\nRoll Tracker +20 degrees then press [RETURN]:"); fflush(stdout);
  while (getch() != CR);

  ts = sendTrackerCMD(&td,"S",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  printf("\nRoll Tracker -20 degrees then press [RETURN]:"); fflush(stdout);
  while (getch() != CR);

  ts = sendTrackerCMD(&td,"S",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
	ts = resetTracker(&td,RTO);
	printf("%s\n",trackerInfo(ts));
	return;
  } // if

  printf("\nTracker Gravimetric Calibration completed.\n");

} // doGravimetricCal

void doCal(void) {
long status,mode;

  printf("\n\nTracker Calibration:\n");
  printf("  1 = Magnetic\n  2 = Gravimetric\n");
  status = getDigitRange(1,2,&mode);
  if (status == 1) {
	switch (mode) {
	  case 1: doMagneticCal(); break;
	  case 2: doGravimetricCal(); break;
	} // switch
  } // if

} // doCal

#define MAX_FILTER 7

void setFiltersMagGrv(void) {
long status,filter,val;
  while (1) {
	printf("\n\nSet Tracker Magnetic or Gravimetric Filters:\n");
	printf("\n[Current %s]\n",getFiltersStr(&td));
	printf("  1 = Magnetic\n  2 = Gravimetric\n");
	status = getDigitRange(1,2,&filter);
	if (status == 1) {
	  printf("Filter Value:\n");
	  status = getDigitRange(0,MAX_FILTER,&val);
	  if (status == 1) {
	switch (filter) {
	  case 1: td.magFilts = val;
		ts = writeTrackerMagFilts(&td);
		if (ts == TS_OK) {
		  printf("\nMagnetic Filters Written ok.\n\n",val);
		} else {
		  printf("%s\n",trackerInfo(ts));
		} // if
		break;
	  case 2: td.grvFilts = val;
		ts = writeTrackerGrvFilts(&td);
		if (ts == TS_OK) {
		  printf("\nGravimetric Filters Written ok.\n\n",val);
		} else {
		  printf("%s\n",trackerInfo(ts));
		} // if
		break;
	} // switch
	  } else if (status == -1) {
	return;
	  } // if
	} else {
	  return;
	} // if
  } // while
} // setFiltersMagGrv

void showTrackerVars(void) {
  printf("\n\nTracker Variables:\n%s\n",getAllVarsStr(&td));
} // showTrackerVars

void showTitle(void) {
  printf("\n\nTCAL: Tracker Calibration Tool. Version %s (%s | %s)\n\n",VERSION,__DATE__,__TIME__);
} // showTitle

void showKeys(void) {
  showTitle();
  printf("Request Data  [F1]\n");
  printf("Calibrate     [F2]\n");
  printf("Set Filters   [F3]\n");
  printf("Show Vars     [F4]\n");
  printf("Run TV        [T]\n");
  printf("Run eulerTV   [E]\n");
  printf("Run Grid      [G]\n");
  printf("Run eulerGrid [M]\n");
  printf("Quit          [Q]\n");
  printf(":"); fflush(stdout);
} // showKeys

void userInput(void) {
TrackerStatus ts;
long doPrompt=1;
long quit = 0;

  while (!quit) {
	long ch;

	if (doPrompt) {
	  doPrompt = 0;
	  showKeys();
	} // if

	ch = getch();

//printf("\n0x%x\n",ch);

	switch (ch) {
	  case CR:
	  case 0: break;
	  case F1:
		doRequestData();
		doPrompt = 1;
		break;
	  case F2:
		doCal();
		doPrompt = 1;
		break;
	  case F3:
		setFiltersMagGrv();
		doPrompt = 1;
		break;
	  case F4:
		ts = readTrackerAllVars(&td);
		if (ts != TS_OK) {
		  printf("%s.\n",trackerInfo(ts));
		} // if
		showTrackerVars();
		doPrompt = 1;
		break;
	  case 't':
	  case 'T':
		printf("\nRunning TV\n");
//        delay(1);
//        flushSer(td.sd);
		startTV(0);
//        delay(1);
//        flushSer(td.sd);
		doPrompt = 1;
		break;
	  case 'e':
	  case 'E':
		printf("\nRunning TV in microEuler\n");
//        delay(1);
//        flushSer(td.sd);
		startTV(1);
//        delay(1);
//        flushSer(td.sd);
		doPrompt = 1;
		break;
	  case 'g':
	  case 'G':

		flushSer(td.sd);

		ts = sendTrackerCMD(&td,"!M1,P,B\r",CTO);
		if (ts != TS_OK) {
		  printf("%s\n",trackerInfo(ts));
		  break;
		} // if

		requestTrackerData(&td);

        gridDemo(&td,1,GRID_MODE_COOKED,12,SHL(20),1);

		doPrompt = 1;
		break;
      case 'm':
      case 'M':

		flushSer(td.sd);

        ts = sendTrackerCMD(&td,"!M2,P,B\r",CTO);
		if (ts != TS_OK) {
		  printf("%s\n",trackerInfo(ts));
		  break;
		} // if

		requestTrackerData(&td);

        gridDemo(&td,1,GRID_MODE_MICRO_EULER,12,SHL(20),1);

		doPrompt = 1;
		break;
      case 'q':
	  case 'Q':
	  case ESC:
		quit = 1;
		break;
	  default:
		break;
	} // switch
  } // while

} // userInput

#define CP(p) checkParm(argc,argv,p)

long checkParm(long argc,char ** argv,char * parm) {
long i;
  for (i=1; i < argc; i++) if (strcmpi(argv[i],parm)==0) return i;
  return 0; // Parm not found.
} // checkParm

void main(int argc,char ** argv) {
TrackerPort trackerPort;
long index;

  showTitle();

  if (CP("-?") || CP("/?") || CP("?") || CP("-h") || CP("/h") || CP("h")) {
	printf("Usage: %s [-debug] [-a 0xXXXX].\n",argv[0]);
	printf("Where 0xXXXX is the hex base address: -a 0x1234.\n");
	exit(0);
  } // if

  debugTrackerIO = CP("-debug") || CP("-d");

  if (debugTrackerIO) {
	printf("Tracker base address: 0x%lx.\n",getBaseAddr());
  } // if

  if (index = CP("-a")) {
	long addr;
	if (sscanf(argv[index+1],"0x%lx",&addr) < 1) {
	  printf("Missing base address for -a command.\n");
	  printf("%s is invalid. Preceed hex number with 0x, as in 0x1234.\n",
	    argv[index+1]);
	  exit(1);
	} // if
	printf("Old Tracker base address:         0x%04lx.\n",getBaseAddr());
	printf("Using new Tracker base address at 0x%04lx.\n",addr);
	setBaseAddr(addr);
  } // if

  if (debugTrackerIO) { printf("InitTracker:"); fflush(stdout); }

  ts = initTrackerSearch(&td,&trackerPort,19200,timerSecs(3));
  if (ts != TS_OK) {
	printf("%s.\n",trackerInfo(ts));
	closeTracker(&td);
	exit(0);
  } // if

  if (debugTrackerIO) {
	printf("%s.\n",trackerInfo(ts));
  } // if

  ts = sendTrackerCMD(&td,"!M2,P,B\r",CTO);
  if (ts != TS_OK) {
	printf("%s\n",trackerInfo(ts));
  } // if

  if (debugTrackerIO) {
	printf("ReadAllVars:"); fflush(stdout);
  } // if

  ts = readTrackerAllVars(&td);
  if (ts != TS_OK) {
	printf("%s.\n",trackerInfo(ts));
  } // if

  if (debugTrackerIO) {
	printf("Debug: %s.\n",trackerInfo(ts));
  } // if

  showTrackerVars();

  userInput();

  closeTracker(&td);

} // main


