/* tv.c */

#include <stdio.h>
#include <math.h>
#include "imath1.h"
#include "vector1.h"
#include "tracker1.h"
#include "gfx1.h"

#define LARROW 0x4b
#define RARROW 0x4d
#define UARROW 0x48
#define DARROW 0x50
#define F1 0x3b
#define F2 0x3c
#define F3 0x3d
#define F4 0x3e
#define F5 0x3f
#define F6 0x40

#define TWOPI (M_PI*2.)

#define MM 10

typedef struct {float x,y,z;} fvec3;
typedef struct {fvec3 c,o;} lvec;
typedef struct {
  float min, max, cur;
} angle;
typedef struct {
  int cx, cy, rad;
  angle n, o;
} dial;

int visible = 0;
int erase = 1;
gfxConfig gc;

void drawDial(dial *d){
  writeFillCircleColor( d->cx, d->cy, d->rad, 7);
  writeCircleColor( d->cx, d->cy, d->rad, 15);
}

void initDial(dial * d,int cx,int cy,int rad,int min,int max) {
  d->cx  = cx;
  d->cy  = cy;
  d->rad = rad;
  d->n.min  = min;
  d->n.max  = max;
  d->n.cur = min;
  drawDial(d);
} // initDial

void updateDial(dial *u){
  float x, y, ang;
  ang = u->o.cur / (u->o.max - u->o.min) * TWOPI;
  x = cos( ang ) * u->rad;
  y = sin( ang ) * u->rad;
  writeLineColor( u->cx, u->cy, u->cx + (int)x, u->cy + (int)y, 7);
  ang = u->n.cur / (u->n.max - u->n.min) * TWOPI;
  x = cos( ang ) * u->rad;
  y = sin( ang ) * u->rad;
  writeLineColor( u->cx, u->cy, u->cx + (int)x, u->cy + (int)y, 4);
  u->o = u->n;
}

void setDial(dial * d,float curr) {
  d->n.cur = curr;
  updateDial(d);
} // setDial

void rotateXYZ( float *x, float *y, float *z, float yaw, float pitch, float roll){
 double fx, fy, fz;
 fx = sin( roll ) * (*y) + cos( roll ) * (*x);
 fy = cos( roll ) * (*y) - sin( roll ) * (*x);
 fz = cos( pitch ) * (*z) - sin( pitch ) * fy;
 *y = sin( pitch ) * (*z) + cos( pitch ) * fy;
 *x = cos( yaw ) * fx - sin( yaw ) * fz;
 *z = sin( yaw ) * fx + cos( yaw ) * fz;
}

void drawProjections(lvec * v){
// scale vector to fit window
  v->c.x /= 303.0;
  v->c.y /= 303.0;
  v->c.z /= 303.0;

	if( visible && erase){

// clear projection of vector on x/y plane
	writeLineColor( 106, 120, 106 + (int)v->o.x, 120 + (int)v->o.y, 7);
// clear projection of vector on x/z plane
	writeLineColor( 320, 120, 320 + (int)v->o.x, 120 + (int)v->o.z, 7);
// clear projection of vector on y/z plane
	writeLineColor( 531, 120, 531 + (int)v->o.y, 120 + (int)v->o.z, 7);

	} // if

// draw new grid
	writeLineColor( 1, 120, 212, 120, 14);
	writeLineColor( 106, 1, 106, 239, 14);
	writeLineColor( 214, 120, 426, 120, 14);
	writeLineColor( 320, 1, 320, 239, 14);
	writeLineColor( 428, 120, 638, 120, 14);
	writeLineColor( 531, 1, 531, 239, 14);

  if( visible ){

// draw projection of vector on x/y plane
	if (abs(v->c.z) < MM)
	  writeLineColor( 106, 120, 106 + (int)v->c.x, 120 + (int)v->c.y, 4);
// draw projection of vector on x/z plane
	if (abs(v->c.y) < MM)
	  writeLineColor( 320, 120, 320 + (int)v->c.x, 120 + (int)v->c.z, 4);
// draw projection of vector on y/z plane
	if (abs(v->c.x) < MM)
	  writeLineColor( 531, 120, 531 + (int)v->c. y, 120 + (int)v->c.z, 4);
  }

// draw pixels for trails
  writePixelColor( 108 + (int)v->c.x, 122 + (int)v->c.y, 1);
  writePixelColor( 322 + (int)v->c.x, 122 + (int)v->c.z, 1);
  writePixelColor( 533 + (int)v->c.y, 122 + (int)v->c.z, 1);

  v->o = v->c;
}

#if 0
void drawMenu(){
  drawWindow( 0, gc.height / 24, gc.width / 32, gc.height / 6, 1);
  drawWindow( 0, gc.height / 3, gc.width / 32, gc.height / 6, 1);
  drawWindow( 0, gc.height / 1.71428, gc.width / 32, gc.height / 6, 1);
  drawWindow( 0, gc.height / 1.2, gc.width / 32, gc.height / 6, 1);
  drawWindow( gc.width / 5.3, gc.height / 24, gc.width / 32, gc.height / 6, 1);
  drawWindow( gc.width / 5.3, gc.height / 3, gc.width / 32, gc.height / 6, 1);
  drawWindow( gc.width / 5.3, gc.height / 1.71428, gc.width / 32, gc.height / 6, 1);
  drawWindow( gc.width / 5.3, gc.height / 1.2, gc.width / 32, gc.height / 6, 1);
  drawWindow( gc.width / 2, 0, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 2, gc.height / 8, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 2, gc.height / 4, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 2, gc.height / 2.6, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 1.3, 0, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 1.3, gc.height / 8, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 1.3, gc.height / 4, gc.width / 4, gc.height / 8, 1);
  drawWindow( gc.width / 1.3, gc.height / 2.6, gc.width / 4, gc.height / 8, 1);

}
#endif

void drawBackground(void) {

  drawWindow( 0, 0, 213, 240, 1);
  drawWindow( 214, 0, 213, 240, 1);
  drawWindow( 428, 0, 212, 240, 1);
  writeTextColor( 0, 240, "x/y plane", 1);
  writeTextColor( 214, 240, "x/z plane", 1);
  writeTextColor( 428, 240, "y/z plane", 1);

} // drawBackground

void getOffsets(TrackerData * td) {
TrackerStatus ts;
  ts = readTrackerOffsets(td);
} // getOffsets

void displayOffsets(TrackerData * td) {
  outText(0,20,getOffsetsStr(td),C_WHITE);
} // displayOffsets

void displayGains(TrackerData * td) {
  outText(0,21,getGainsStr(td),C_WHITE);
} // displayGains

void displayFilters(TrackerData * td) {
  outText(0,22,getFiltersStr(td),C_WHITE);
} // displayFilters

void displayMinMax(TrackerData * min,TrackerData * max) {
long dx,dy,dz;
static char s[80];
  dx = max->magnetic.x + min->magnetic.x;
  dy = max->magnetic.y + min->magnetic.y;
  dz = max->magnetic.z + min->magnetic.z;
  sprintf(s,"Min x %5d y %5d z %5d",min->magnetic.x,min->magnetic.y,min->magnetic.z);
  outText(0,23,s,C_WHITE);
  sprintf(s,"Max x %5d y %5d z %5d",max->magnetic.x,max->magnetic.y,max->magnetic.z);
  outText(0,24,s,C_WHITE);
  sprintf(s,"Del x %5d y %5d z %5d",dx,dy,dz);
  outText(0,25,s,C_WHITE);
} // displayMinMax

#define IMIN (1<<29)  
#define IMAX (-IMIN)

void resetMinMax(TrackerData * min,TrackerData * max) {
  min->magnetic.x = IMIN;
  min->magnetic.y = IMIN;
  min->magnetic.z = IMIN;
  max->magnetic.x = IMAX;
  max->magnetic.y = IMAX;
  max->magnetic.z = IMAX;
} // resetMinMax

#define UMAG(a,b,cmp,v) if ((a)->magnetic.v cmp (b)->magnetic.v) \
  {(b)->magnetic.v = (a)->magnetic.v; update = 1;}

void updateMinMax(TrackerData * curr,TrackerData * min,TrackerData * max) {
long update;  
  update = 0;
  UMAG(curr,min,<,x);    
  UMAG(curr,min,<,y);    
  UMAG(curr,min,<,z);    
  UMAG(curr,max,>,x);    
  UMAG(curr,max,>,y);    
  UMAG(curr,max,>,z);    
  if (update) {
	displayMinMax(min,max);
  } // if
} // updateMinMax

void startTV(long smode) {
  GraphicsStatus gs;
  int mode;
  lvec magnetic;
  char string[64];
  float yaw = 0, pitch = 0, roll = 0;
  dial yawDial,pitchDial,rollDial;
  dial xDial,yDial,zDial;
  long mlo=1<<29,mhi=-1,mdel=0,mcurr=0;

  extern TrackerData td; // in tcal.c
  
  TrackerData minTD,maxTD;
  TrackerStatus ts;

  resetMinMax(&minTD,&maxTD);

  ts = resetTracker(&td,timerSecs(2));
  if (ts != TS_OK) {
	printf("%s.\n",trackerInfo(ts));
	goto done;
  } // if

  if (smode == 0) { // cooked
	ts = sendTrackerCMD(&td,"!M1,P,B\r",timerTSecs(20));
	if (ts != TS_OK) {
	  printf("%s.\n",trackerInfo(ts));
	  goto done;
	} // if
  } else {
	ts = sendTrackerCMD(&td,"!M2,P,B\r",timerTSecs(20));
	if (ts != TS_OK) {
	  printf("%s.\n",trackerInfo(ts));
	  goto done;
	} // if
  } // if

#if 1
  ts = readTrackerAllVars(&td);
  if (ts != TS_OK) {
	printf("%s.\n",trackerInfo(ts));
	goto done;
  } // if
#endif

  magnetic.o.x = 0;
  magnetic.o.y = 0;
  magnetic.o.z = 0;

  gs = initGraphics( &gc, GM_640x480x16);

  switch( gs ){
	case GS_OK:
//        printf( "Initialized Graphics Ok\n" );
	  break;
	case GS_NA:
	  printf( "Graphics Mode Not Available\n" );
	  goto done;
  }

  initDial(&yawDial,50,480-32-3,32,0.,360.);

  initDial(&pitchDial,50+100,480-32-3,32,-180.,180.);

  initDial(&rollDial,50+100*2,480-32-3,32,-180.,180.);


  initDial(&xDial,320+50,480-32-3,32,-16484,16384);

  initDial(&yDial,320+50+100,480-32-3,32,-16384,16384);

  initDial(&zDial,320+50+100*2,480-32-3,32,-16384,16384);


  drawBackground();

#if 1
  getOffsets(&td);
  displayOffsets(&td);
  displayGains(&td);
  displayFilters(&td);
#endif

  requestTrackerData(&td);

  while (1) {

	if (smode == 0) {
	  ts = readTracker(&td,timerSecs(1));
	  if (ts != TS_OK && ts != TS_TIMEOUT && ts != TS_CHECKSUM) {
	printf("\n%s.\n",trackerInfo(ts));
	goto done;
	  } // if
	} else {
	  ts = readTrackerEuler(&td,timerSecs(1));
	  if (ts != TS_OK && ts != TS_TIMEOUT && ts != TS_CHECKSUM) {
	printf("\n%s.\n",trackerInfo(ts));
	goto done;
	  } // if
	} // if

	if (ts != TS_OK) {
//      flushSer(td.sd);
	  ts = resetTracker(&td,timerSecs(2));
	  printf("%s.\n",trackerInfo(ts));
	}

//    updateMinMax(&td,&minTD,&maxTD);

	magnetic.c.x = td.magnetic.x;
	magnetic.c.y = td.magnetic.y;
	magnetic.c.z = td.magnetic.z;
	mcurr = sqrt(magnetic.c.x*magnetic.c.x + magnetic.c.y*magnetic.c.y + magnetic.c.z*magnetic.c.z);
	if (mcurr < mlo) mlo = mcurr;
	if (mcurr > mhi) mhi = mcurr;
	mdel = mhi - mlo;

//      rotateXYZ( &magnetic.c.x, &magnetic.c.y, &magnetic.c.z, yaw, pitch, roll);

	drawProjections( &magnetic );

//    writeFillBoxColor( 0, 270, 640, 290, 0);
//    sprintf(string,"Lo %5d Hi %5d Delta %5d Curr %5d",
//            mlo,mhi,mdel,mcurr);

	yaw   = TOFLOAT(td.euler.y);
	pitch = TOFLOAT(td.euler.x);
	roll  = TOFLOAT(td.euler.z);

	sprintf(string,"Yaw %6.2f Pitch %6.2f Roll %6.2f",yaw,pitch,roll);
	outText(0,25,string,C_WHITE);

// Dials
	setDial(&yawDial,yaw);
	setDial(&pitchDial,pitch);
	setDial(&rollDial,roll);

	sprintf(string,"x %6d     y %6d    z %6d",td.magnetic.x,td.magnetic.y,td.magnetic.z);
	outText(43,25,string,C_WHITE);

// More dials
	setDial(&xDial,td.magnetic.x);
	setDial(&yDial,td.magnetic.y);
	setDial(&zDial,td.magnetic.z);

	if (kbhit()) {
	  char ch = getch();
//printf("%x\n",ch);
	  switch (ch) {
	case ' ': visible ^= 1; break;
	case 0xd: drawBackground(); 
//          resetMinMax(&minTD,&maxTD);
	  break;
	case 'q':
	case 'Q':
	case 27 : goto done;
	case 'e': erase ^= 1; break;
#if 0        
	case '1': td.magOff.x -= 10;
	  ts = writeTrackerOffset(&td,TA_X);
	  displayOffsets(&td);
	  break;
	case '2': td.magOff.x += 10;
	  ts = writeTrackerOffset(&td,TA_X);
	  displayOffsets(&td);
	  break;
	case '3': td.magOff.y -= 10;
	  ts = writeTrackerOffset(&td,TA_Y);
	  displayOffsets(&td);
	  break;
	case '4': td.magOff.y += 10;
	  ts = writeTrackerOffset(&td,TA_Y);
	  displayOffsets(&td);
	  break;
	case '5': td.magOff.z -= 10;
	  ts = writeTrackerOffset(&td,TA_Z);
	  displayOffsets(&td);
	  break;
	case '6': td.magOff.z += 10;
	  ts = writeTrackerOffset(&td,TA_Z);
	  displayOffsets(&td);
	  break;
//*** Gains ***
	case F1: td.magGain.x -= 10;
	  ts = writeTrackerGain(&td,TA_X);
	  displayGains(&td);
	  break;
	case F2: td.magGain.x += 10;
	  ts = writeTrackerGain(&td,TA_X);
	  displayGains(&td);
	  break;
	case F3: td.magGain.y -= 10;
	  ts = writeTrackerGain(&td,TA_Y);
	  displayGains(&td);
	  break;
	case F4: td.magGain.y += 10;
	  ts = writeTrackerGain(&td,TA_Y);
	  displayGains(&td);
	  break;
	case F5: td.magGain.z -= 10;
	  ts = writeTrackerGain(&td,TA_Z);
	  displayGains(&td);
	  break;
	case F6: td.magGain.z += 10;
	  ts = writeTrackerGain(&td,TA_Z);
	  displayGains(&td);
	  break;
#endif      
	  } // switch
	} // if
  
	requestTrackerData(&td);
  
  } // while

done:  
  
  CloseGraphics();

} // startTV

