/* Tracker.c
   Tracker program for setting various mouse modes.
   Created 2/17/95.
   Compiled under Watcom 10.0a, DOS4G/W 1.97.
   Written by John Schwab
   Modified 2/28/95 jcs. Firmware bug work-arounds added.
*/

#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "imath1.h"
#include "vector1.h"
#include "convert1.h"
#include "tracker1.h"
#include "serial1.h"
#include "tracker.h"

BYTE tport = 0;
char verstr[TRACKER_VER_LEN];
short scharwidth = 80, scharheight = 50; //screen character width and height
short curwin = HELP, lastwin = HELP;
long mode, port = 0;
TrackerData td={0}; //tracker data structures
TrackerStatus ts;

char helpkeys[] = {
	 "ACTION             KEY PRESS\n"
	 "============================\n"
	 "Select a setting:\n"
	 "            TAB or SHIFT-TAB\n"
	 "Change a setting:\n"
	 "         LEFT or RIGHT ARROW\n"
	 "Save new settings and exit:\n"
	 "                       ENTER\n"
	 "Cancel and exit:\n"
	 "                      ESCAPE\n"
	 };
/*-----Help menu definitions---------*/
char *helplist[5] = {
	{"           HELP\n\n"
	 "Welcome to TRACKER, the\n"
	 "Virtual i-O Tracker Manager.\n"
	 "This program lets you play\n"
     "games with the Tracker\n"
     "instead of with a mouse.\n\n"

     "You can also use TRACKER\n"
     "to set filters for games\n"
     "that support the Tracker\n"
     "directly (set emulation to\n"
     "NONE).\n\n"
//     "You don't have to use\n"
//     "TRACKER for games that were\n"
//     "written to use a head\n"
//     "tracker.\n\n"
     "Use TRACKER to set up an\n"
     "emulation mode, then play\n"
     "the mouse controlled game.\n"
     "The Tracker remembers how\n"
     "you used it last, so you\n"
     "only have to use TRACKER\n"
     "when switching between\n"
     "different emulations.\n"
     "To the left of this screen\n"
	 "are the current settings in\n"
     "the Tracker which you\n"
	 "can change.\n"},
	{"        EMULATION\n\n"
     "The Tracker can emulate\n"
     "either a mouse or a mouse\n"
     "acting as a joystick.\n\n"
	 "If the game you want to play\n"
	 "normally uses a mouse, then\n"
     "set the Tracker to emulate a\n"
     "MOUSE.\n\n"
     "If the game uses a mouse\n"
     "that acts like a joystick,\n"
     "use JOYSTICK emulation.\n\n"
     "If the game directly\n"
     "supports the Tracker, set\n"
     "the Tracker to emulate NONE.\n"
	 "This will allow setting of\n"
     "the filters only.\n\n"},
    {"          FILTERS\n\n"
     "The Tracker can filter\n"
	 "\"jitter\" out of your head\n"
	 "movements. Filtering makes\n"
	 "the tracker run smoother\n"
     "but slower. It has one\n"
     "filter for turning (Turn),\n"
     "and one for tilting (Tilt).\n\n"
	 "The normal filter settings\n"
     "are 3 and 3. A 0 setting\n"
     "is no filtering.\n\n"
	 "You probably want to have\n"
	 "the same settings on both\n"
	 "filters.\n\n"
	 "If you want to increase the\n"
	 "speed of the tracker\n"
	 "response, decrease the\n"
	 "filter setting.\n\n"
	 "If you want to smooth out\n"
	 "the tracker response,\n"
	 "increase the filter\n"
	 "setting.\n\n"},
	{"       SENSITIVITY\n\n"
	 "When emulating a mouse or\n"
     "mouse-joystick, the Tracker\n"
	 "can reduce or magnify the\n"
	 "effect of moving your head.\n"
	 "It has one sensitivity for\n"
	 "turning (X), and one for\n"
	 "tilting (Y).\n\n"
	 "The normal sensitivity\n"
	 "settings are 1 and 3. A 0\n"
	 "sensitivity turns off the\n"
     "Tracker response in that\n"
	 "direction.\n\n"
	 "You probably want to have\n"
	 "the Y sensitivity setting\n"
	 "two more then that of X.\n\n"
	 "If turning your head all\n"
	 "the way behind you doesn't\n"
	 "look straight back in the\n"
	 "game, then change the\n"
	 "turning (X) sensitivity.\n\n"
	 "If tilting your head all\n"
	 "the way up doesn't look\n"
	 "straight up in the game,\n"
	 "then change the tilting\n"
	 "(Y) sensitivity.\n\n"},
	{"        THRESHOLD\n\n"
	 "When emulating a mouse or\n"
     "mouse-joystick, the Tracker\n"
	 "can respond to small or\n"
	 "large movements of your\n"
	 "head. It has one threshold\n"
	 "for turning (X), and one\n"
	 "for tilting (Y).\n\n"
	 "The normal threshold\n"
	 "settings are 2. A 0\n"
	 "threshold responds to any\n"
	 "movement in that direction.\n"
	 "You probably want to have\n"
	 "the same threshold setting\n"
	 "in both directions\n"
	 "(X and Y).\n\n"},
};

/*-----Screen button definitions-----*/
BUTTON jem = {19,30,3 ,5 ,YELLOW,BLUE,WHITE,"Emulation"},
	   fil = {20,29,16,18,YELLOW,BLUE,WHITE,"Filters"},
	   sen = {8 ,21,29,31,YELLOW,BLUE,WHITE,"Sensitivity"},
	   thr = {31,42,29,31,YELLOW,BLUE,WHITE,"Threshold"},
	   hlp = {62,68,2 ,4 ,YELLOW,RED ,CYAN ,"Help"};

/*-----Screen slider definitions-----*/
SLIDE emu = {15,33,7 ,9 ,YELLOW,CYAN,RED,0},
	  ftx = {9 ,18,20,22,YELLOW,CYAN,RED,1},
	  fty = {31,40,20,22,YELLOW,CYAN,RED,5},
	  snx = {9 ,20,33,35,YELLOW,CYAN,RED,1},
	  sny = {9 ,20,37,39,YELLOW,CYAN,RED,4},
	  thx = {31,42,33,35,YELLOW,CYAN,RED,2},
	  thy = {31,42,37,39,YELLOW,CYAN,RED,2};

/*-----Set a 80x50 text mode-----*/
void set_mode50(){
 union REGS regs;
 regs.w.ax = 0x500;             //set active page to 0
 int386( VIDEO_INT, &regs, &regs); //call the interrupt
 regs.w.ax = 0x1202;            //set 400 scan lines
 regs.h.bl = 0x30;
 int386( VIDEO_INT, &regs, &regs);
 regs.h.ah = 0x0f;              //determine the current video mode
 int386( VIDEO_INT, &regs, &regs);
 regs.h.ah = SET_VMODE;         //set video mode
 int386( VIDEO_INT, &regs, &regs);
 regs.w.ax = 0x1112;            //set to 8x8 character set
 regs.h.bl = 0;                 //set block number to 0
 int386( VIDEO_INT, &regs, &regs);
 regs.h.ah = 1;                 //set cursor position
 regs.w.cx = 0x600;             //to start at 6 and end at 0
 int386( VIDEO_INT, &regs, &regs);
 outp( CRTC_ADDRESS, UNDERLINE_LOC);
 outp( CRTC_DATA, 7);           //set underline to line 7
}

/*-----Set Bios 80x25 mode-----*/
void set_mode25(){
 union REGS regs;
 regs.h.ah = SET_VMODE; //set video mode
 regs.h.al = MODE80X25; //set to bios mode 3 (80x25)
 int386( VIDEO_INT, &regs, &regs);
}

/*-----Write a character directly to video----*/
void put_char( int x, int y, BYTE chr, BYTE attr){
 short *cvid;
 cvid = (short *)(((y << 7) + (y << 5)) + (x << 1) + 0xb8000);
 *cvid = (short)((attr << 8)|chr);
}

/*-----Write a string to video using new line a termination-----*/
void put_string( int x, int y, BYTE *str, BYTE attr){
 int sx = x;
 while( *str ){
  if( *str == '\n'){
   sx = x;
   y++;
  }
  else{
   put_char( sx++, y, *str, attr);
  }
  str++;
 }
}

/*----Draw a text box filled-----*/
void draw_box( int x1, int y1, int x2, int y2, BYTE chr, BYTE fclr, BYTE bclr, BYTE blnk){
 short attrib;
 int x, y, d;
 short *vidf;
 attrib = (blnk << 15)|(bclr << 12)|(fclr << 8) | chr;
 if( x1 > x2)   SWAP( x1, x2);
 if( y1 > y1)   SWAP( y1, y2 );
 vidf = (short *)(((y1 << 7) + (y1 << 5)) + (x1 << 1) + 0xb8000);
 for( y = y1; y <= y2; y++){
  for( x = x1; x <= x2; x++){
   *vidf++ = attrib;
  }
  vidf += 79 - (x2 - x1);
 }
}

/*----draw a text box outline----*/
void draw_win( int x1, int y1, int x2, int y2, BYTE chr, BYTE fclr, BYTE bclr, BYTE blnk){
 short attrib;
 int x, y, d;
 short *vidt, *vidb;
 attrib = (blnk << 15)|(bclr << 12)|(fclr << 8) | chr;
 if( x1 > x2) SWAP( x1, x2);
 if( y1 > y1) SWAP( y1, y2);
 vidt = (short *)(((y1 << 7) + (y1 << 5)) + (x1 << 1) + 0xb8000);
 vidb = (short *)(((y2 << 7) + (y2 << 5)) + (x1 << 1) + 0xb8000);
 for( x = x1; x <= x2; x++){
  *vidt++ = attrib;
  *vidb++ = attrib;
 }
 vidt = (short *)(((y1 << 7) + (y1 << 5)) + (x1 << 1) + 0xb8000);
 vidb = (short *)(((y1 << 7) + (y1 << 5)) + (x2 << 1) + 0xb8000);
 for( y = y1+1; y < y2; y++){
  *(vidt += 80) = attrib;
  *(vidb += 80) = attrib;
 }
}

/*-----Draw a button----*/
void draw_button( BUTTON *bt){
 put_string( bt->x1+1, bt->y1+1, bt->btext, (bt->bclr<<4)|bt->tclr);
}

/*----draw a slider bar with value printed----*/
void draw_slide( SLIDE *sl){
 draw_win( sl->x1, sl->y1, sl->x2, sl->y2, BORDER_CHAR, sl->fclr, sl->bclr, 0);
 draw_box( sl->x1+1, sl->y1+1, sl->x2-1, sl->y2-1, FILL_CHAR, sl->bclr, sl->bclr, 0);
 put_char( sl->x1+sl->sval+1, sl->y1+1, ITOC(sl->sval), (RED<<4)|BRIGHTWHITE);
}

/*----draw all mouse settings----*/
void draw_mouse_sets(){
 draw_button( &sen );
 draw_button( &thr );
 draw_slide( &snx );
 draw_slide( &sny );
 draw_slide( &thx );
 draw_slide( &thy );
 put_string( snx.x1-2, snx.y1+1, "X", (BLUE<<4)|WHITE);
 put_string( sny.x1-2, sny.y1+1, "Y", (BLUE<<4)|WHITE);
 put_string( thx.x1-2, thx.y1+1, "X", (BLUE<<4)|WHITE);
 put_string( thy.x1-2, thy.y1+1, "Y", (BLUE<<4)|WHITE);

}

/*----draw emulation setting strings----*/
void draw_emulate( long val){
 draw_slide( &emu );
 put_string( emu.x1+1, emu.y1+1, "", emu.bclr);
 if( val <= 0){
  put_string( emu.x1+1, emu.y1+1, "None", (RED<<4)|BRIGHTWHITE);
 }
 else if( val == 1){
  put_string( emu.x1+5, emu.y1+1, "Mouse", (RED<<4)|BRIGHTWHITE);
 }
 else{
  put_string( emu.x1+10, emu.y1+1, "Joystick", (RED<<4)|BRIGHTWHITE);
 }
 if( emu.sval != 0) draw_mouse_sets();
}

void draw_help( int win){
 switch( win ){
  case HELP:
   put_string( 51, 2, helplist[ 0 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case EMULATION:
   put_string( 51, 2, helplist[ 1 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case XFILTER:
   put_string( 51, 2, helplist[ 2 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case YFILTER:
   put_string( 51, 2, helplist[ 2 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case XSENSITIVITY:
   put_string( 51, 2, helplist[ 3 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case YSENSITIVITY:
   put_string( 51, 2, helplist[ 3 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case XTHRESHOLD:
   put_string( 51, 2, helplist[ 4 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
  case YTHRESHOLD:
   put_string( 51, 2, helplist[ 4 ], (hlp.bclr<<4)|BRIGHTWHITE);
   break;
 }
}

/*----draw all screen initializing settings----*/
void draw_screen(){
 put_string( 0, 0, "Tracker Manager 2.0     Copyright 1995 "
				   "Virtual i-O Inc.                            ", (CYAN<<4)|BRIGHTWHITE); draw_box( 1, 2, 78, 48, FILL_CHAR, BLUE, BLUE, 0);
 draw_win( 0, 1, 79, 49, BORDER_CHAR, GRAY, BLUE, 0);
 draw_win( 50, 1, 79, 49, BORDER_CHAR, hlp.fclr, hlp.bclr, 0);
 draw_box( 51, 2, 78, 48, FILL_CHAR, hlp.bclr, hlp.bclr, 0);
 draw_button( &jem );
 draw_button( &fil );
 draw_slide( &emu );
 draw_slide( &ftx );
 draw_slide( &fty );
 put_string( ftx.x1-2-3, ftx.y1+1, "Turn", (BLUE<<4)|WHITE);
 put_string( fty.x1-2-3, fty.y1+1, "Tilt", (BLUE<<4)|WHITE);
 draw_help( curwin );
 put_string( 51, 38, &helpkeys, (hlp.bclr<<4)|BRIGHTWHITE);
 put_string( 2, 47, verstr, (BLUE<<4)|BRIGHTWHITE);
 draw_emulate( emu.sval );
}

/*----get physical screen height----*/
int get_scr_char_height(){
 BYTE *bios_rps = (BYTE *)0x484;        //address for bios rows per screen.
 return( *bios_rps );
}

/*----Update the current window selected----*/
void update_window( int win, int dir){
 BYTE tclr;
 int twice, cnt = 1;
 if( !dir ){
  win = lastwin;
  cnt = 2;
 }
 do{
  switch( win ){
   case HELP:
	if( !dir ) SWAP( hlp.tclr, hlp.bclr);
	draw_win( 50, 1, 79, 49, BORDER_CHAR, hlp.fclr, hlp.bclr, 0);
	draw_box( 51, 2, 78, 48, FILL_CHAR, hlp.bclr, hlp.tclr, 0);
	break;
   case EMULATION:
	if( !dir ){
	 SWAP( emu.bclr, emu.tclr);
	}
	emu.sval += dir;
	if( emu.sval <= 0) emu.sval = 0;
	if( emu.sval >= 2) emu.sval = 2;
	draw_emulate( emu.sval );
	break;
   case XFILTER:
	if( !dir ) SWAP( ftx.bclr, ftx.tclr);
	ftx.sval += dir;
	if( ftx.sval < 0) ftx.sval = 0;
	if( ftx.sval > ftx.x2 - ftx.x1 - 2) ftx.sval = ftx.x2 - ftx.x1 - 2;
	draw_slide( &ftx );
	break;
   case YFILTER:
	if( !dir ) SWAP( fty.bclr, fty.tclr);
	fty.sval += dir;
	if( fty.sval < 0) fty.sval = 0;
	if( fty.sval > fty.x2 - fty.x1 - 2) fty.sval = fty.x2 - fty.x1 - 2;
	draw_slide( &fty );
	break;
  }
  if( emu.sval ){
   draw_mouse_sets();
   switch( win ){
	case XSENSITIVITY:
	 if( !dir ) SWAP( snx.bclr, snx.tclr);
	 snx.sval += dir;
	 if( snx.sval < 0) snx.sval = 0;
	 if( snx.sval > snx.x2 - snx.x1 - 2) snx.sval = snx.x2 - snx.x1 - 2;
	 draw_slide( &snx );
	 break;
	case YSENSITIVITY:
	 if( !dir ) SWAP( sny.bclr, sny.tclr);
	 sny.sval += dir;
	 if( sny.sval < 0) sny.sval = 0;
	 if( sny.sval > sny.x2 - sny.x1 - 2) sny.sval = sny.x2 - sny.x1 - 2;
	 draw_slide( &sny );
	 break;
	case XTHRESHOLD:
	 if( !dir ) SWAP( thx.bclr, thx.tclr);
	 thx.sval += dir;
	 if( thx.sval < 0) thx.sval = 0;
	 if( thx.sval > thx.x2 - thx.x1 - 2) thx.sval = thx.x2 - thx.x1 - 2;
	 draw_slide( &thx );
	 break;
	case YTHRESHOLD:
	 if( !dir ) SWAP( thy.bclr, thy.tclr);
	 thy.sval += dir;
	 if( thy.sval < 0) thy.sval = 0;
	 if( thy.sval > thy.x2 - thy.x1 - 2) thy.sval = thy.x2 - thy.x1 - 2;
	 draw_slide( &thy );
   }
  }
  else draw_box( 5, 25, 45, 40, FILL_CHAR, BLUE, BLUE, 0);
  win = curwin;
 }while( --cnt );
}

/*----detect the com port if a tracker is installed----*/
int detect_tracker(){
 long hver, hrev, fver, frev;
 float hversion, fversion;
 printf("\n\nSearching for a Tracker.\n");
 for( port = TP_COM1; port <= TP_COM4; port++){

// Be sure to call closeTracker before your program exits. It can be
// called even if not successfully opened.

// You may want to run everything at 9600 until you exit, then set
// the tracker to 1200 for mouse mode (if necessary).
// This will make everything run faster for the user.

// Changed to 1200*8=9600 2/22/95 JCS. Changed back 2/23/95 1:45am ////////
  ts = initTracker( &td, port, 1200*8, timerSecs(2));
  if( ts == TS_OK){
   printf( "Tracker found on COM%1d\n", port + 1);
   tport = port + 1;

#if 0
////// Added 2/22/95 JCS ///////////////////
   setBPS(td.sd,1200*4);
   ts = resetTracker(&td,timerSecs(3));
   if (ts != TS_OK ) {
	 printf("%s\n",trackerInfo(ts));
	 closeTracker(&td);
	 return 0;
   } // if
/////////////////////////////////////////////
#endif
//printf("Getch():"); fflush(stdout); getch();
//   delay( 500 );
//   flushSer( td.sd );

   ts = getTrackerVerRev(&td,&hver,&hrev,&fver,&frev,timerSecs(2));
   if (ts != TS_OK) {
	printf("%s.\n",trackerInfo(ts));
	closeTracker(&td);
	exit(0);
   }
   hversion = (float)hver + (float)hrev/1000.;
   fversion = (float)fver + (float)frev/1000.;
   sprintf(verstr, "Hardware Version %4.3f  Firmware Version %4.3f\n",hversion,fversion);
   if( ts != TS_OK){
	printf( "Couldn't read version string:\n%s\n", trackerInfo( ts ));
	return( 0 );
   }
   printf( "%s\n", verstr);
   return( 1 );
  }
  else{
   printf("%s\n",trackerInfo(ts));
   printf( "Tracker not found on COM%1d\n", port + 1);
   closeTracker( &td );
  }
 }
 return( 0 );
}

long get_tracker_current(){
 ts = readTrackerMiscVar( &td, DATA_MODE_OFFSET, &mode);
 if( mode < 0 || mode > 3){
  printf( "Data Mode out of range. %d\n",mode);
  if( mode < 0) mode = 0;
  if( mode > 3) mode = 3;
  getch();
 }
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 ts = readTrackerMiscVar( &td, MOUSE_MODE_OFFSET, &emu.sval);
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 if( emu.sval < 0 || emu.sval > 1){
  printf( "Mouse mode out of range. %d\n",emu.sval);
  if( emu.sval < 0) emu.sval = 0;
  if( emu.sval > 1) emu.sval = 1;
  getch();
 }
 if( mode != 3) emu.sval = 0;
 else emu.sval++;
 ts = readTrackerFilters( &td);
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 ftx.sval = td.magFilts;
 if( ftx.sval < 0 || ftx.sval > 7){
  printf( "X Filter out of range. %d\n", ftx.sval);
  if( ftx.sval < 0) ftx.sval = 0;
  if( ftx.sval > 7) ftx.sval = 7;
  getch();
 }
 fty.sval = td.grvFilts;
 if( fty.sval < 0 || fty.sval > 7){
  printf( "Y Filter out of range. %d\n", fty.sval);
  if( fty.sval < 0) fty.sval = 0;
  if( fty.sval > 7) fty.sval = 7;
  getch();
 }
 ts = readTrackerMiscVar( &td, MOUSE_X_SENSI_OFFSET, &snx.sval);
 if( snx.sval < 0 || snx.sval > 9){
  printf( "X Sensitivity out of range. %d\n", snx.sval);
  if( snx.sval < 0) snx.sval = 0;
  if( snx.sval > 9) snx.sval = 9;
  getch();
 }
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 ts = readTrackerMiscVar( &td, MOUSE_Y_SENSI_OFFSET, &sny.sval);
 if( sny.sval < 0 || sny.sval > 9){
  printf( "Y Sensitivity out of range. %d\n", sny.sval);
  if( sny.sval < 0) sny.sval = 0;
  if( sny.sval > 9) sny.sval = 9;
  getch();
 }
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 ts = readTrackerMiscVar( &td, MOUSE_X_THRESH_OFFSET, &thx.sval);
 if( thx.sval < 0 || thx.sval > 9){
  printf( "X Threshold out of range. %d\n", thx.sval);
  if( thx.sval < 0) thx.sval = 0;
  if( thx.sval > 9) thx.sval = 9;
  getch();
 }
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 ts = readTrackerMiscVar( &td, MOUSE_Y_THRESH_OFFSET, &thy.sval);
 if( thy.sval < 0 || thy.sval > 9){
  printf( "Y Threshold out of range. %d\n", thy.sval);
  if( thy.sval < 0) thy.sval = 0;
  if( thy.sval > 9) thy.sval = 9;
  getch();
 }
 if( ts != TS_OK){
  printf(" %s.\n", trackerInfo( ts ));
  return( 0 );
 }
 return( 1 );
}


TrackerStatus resetTracker3bang(TrackerData * td,long timeout) {
TrackerStatus ts;
long startA,startB;
long ch;
extern long debugTrackerIO;
  startA = getTime();
  flushSer(td->sd);
  while (1) {
	writeSerString(td,"!!!\r"); // will be "!\r" when firmware bugs fixed.
	startB = getTime();
	while (1) {
	  if (!rxBuffEmpty(td->sd)) {
		ch = readSer(td->sd);
		if (debugTrackerIO) printf("\nRead %c\n",ch);
		if (ch == 'O') return TS_OK;
	  } // if
	  if (elapsedTime(startB,timerTSecs(2))) break;
	} // while
	if (elapsedTime(startA,timeout)) return TS_TIMEOUT;
  } // while
} // resetTracker3bang


void main(int argc,char ** argv) {
 char key, cmd[256];
 BYTE tp[2];
 int tinit;
// extern long debugTrackerIO;
// debugTrackerIO = 1;

 if (argc > 1) {
   vtmode("Usage: tracker <args>\n(Use no args for interactive mode)\n",argc,argv);
   exit(0);
 } // if

 if( detect_tracker() == NULL) exit(0);

 get_tracker_current();

 set_mode50();
 scharheight = get_scr_char_height();
 draw_screen();
 put_string( 60, 0, "Tracker on COM",(CYAN<<4)|BRIGHTWHITE);
 tp[0] = ITOC( tport );
 tp[1] = 0;
 put_string( 74, 0, tp, (CYAN<<4)|BRIGHTWHITE);
 do{
  key = getch();
  switch( key ){
   case TAB:
	lastwin = curwin;
	curwin++;
	if( !emu.sval ){
	 if( curwin > YFILTER) curwin = HELP;
	}
	else{
	 if( curwin > YTHRESHOLD) curwin = HELP;
	}
	update_window( curwin, 0);
	draw_box( 51, 2, 78, 48, FILL_CHAR, hlp.bclr, hlp.tclr, 0);
	draw_help( curwin );
	put_string( 51, 38, &helpkeys, (hlp.bclr<<4)|BRIGHTWHITE);
	break;
   case STAB:
	lastwin = curwin;
	curwin--;
	if( !emu.sval){
	 if( curwin < HELP) curwin = YFILTER;
	}
	else{
	 if( curwin < HELP) curwin = YTHRESHOLD;
	}
	update_window( curwin, 0);
	draw_box( 51, 2, 78, 48, FILL_CHAR, hlp.bclr, hlp.tclr, 0);
	draw_help( curwin );
	put_string( 51, 38, &helpkeys, (hlp.bclr<<4)|BRIGHTWHITE);
	break;
   case UP:
	break;
   case DOWN:
	break;
   case LEFT:
	if( curwin != HELP) update_window( curwin, DEC);
	break;
   case RIGHT:
	if( curwin != HELP) update_window( curwin, INC);
	break;
   case ESC:
	set_mode25();
	closeTracker( &td );
	exit(0);
	break;
  }
 }while( key != ENTER);
 set_mode25();

// Changed 2/28/95 jcs: If no emulation selected, set to reset state:
// Cooked, polled, binary. Set new filter values.
// This will allow a way to take the tracker out of mouse mode. This is
// necessary in cases where the user doesn't want the Tracker
// to be mistaken for a mouse by their mouse driver.

 if( emu.sval == 0 ){
//  if( mode == 3) sprintf( cmd, "!M3,0,B,%d,%d\r", ftx.sval, fty.sval);
//  else sprintf(cmd, "!M%d,P,B,%d,%d\r",mode ,ftx.sval ,fty.sval);
  sprintf(cmd,"!M2,P,B,%d,%d\r",ftx.sval,fty.sval);
 }
 else{
  sprintf(cmd,"!M3,%d,B,%d,%d,%d,%d,%d,%d\r",emu.sval-1,ftx.sval ,fty.sval ,
							snx.sval ,sny.sval ,thx.sval ,thy.sval);
 }

 ts = sendTrackerCMD(&td ,cmd ,timerSecs( 3 ));

 if( ts != TS_OK ) {
  printf("%s\n",trackerInfo( ts ));
  closeTracker( &td );
  exit(0);
 }

 if (emu.sval != 0) {
// Set up for mouse mode bps rate.

#if 1
   setBPS(td.sd,1200);
   ts = resetTracker3bang( &td, timerSecs(3));
   printf("%s.\n", trackerInfo( ts ));

   if( ts != TS_OK ) {
	closeTracker( &td );
	exit(0);
   }

#endif

// Start mouse mode stream.
   requestTrackerData(&td);
// Wait for character to be fully sent.
   while (!txBuffEmpty(td.sd));

 } else {
   printf("%s.\n",trackerInfo(ts));
 } // if

 closeTracker( &td );
 exit(0);

}
