/* tracker.h */
/* Created 12/12/94 */
#ifndef TRACKER_H
#define TRACKER_H

#define TRACKER_VER_LEN 60

typedef enum {TP_COM1,TP_COM2,TP_COM3,TP_COM4} TrackerPort;
typedef enum {
  TS_OK,TS_NO_SERIAL,TS_NO_TRACKER,TS_TIMEOUT,
  TS_CHECKSUM,TS_HEADER,TS_WRITEVERIFY,TS_INVALID_PARM,
  TS_CMD_ERROR,TS_NO_MEMORY,TS_INVALID_VER_STR
} TrackerStatus;

typedef struct {
  void * sd;  // Serial Data
  vec3 magnetic;
  vec2 gravimetric;
  vec3 euler;
  vec3 magOff;
  vec3 magGain;
  vec2 grvOff;
  vec2 grvGain;
  long magFilts;
  long grvFilts;
} TrackerData;

extern long debugTrackerIO;

TrackerStatus initTracker(TrackerData * td,TrackerPort port,long comRate,long timeout);
TrackerStatus initTrackerSearch(TrackerData * td,TrackerPort * port,long comRate,long timeout);
void closeTracker(TrackerData * td);
TrackerStatus readPacket(TrackerData * td,char * data,long size,long timeout);
TrackerStatus readTracker(TrackerData * td,long timeout);
TrackerStatus readTrackerEuler(TrackerData * td,long timeout);
void writeSerString(TrackerData * td,char * s);
void requestTrackerData(TrackerData * td);
char * trackerInfo(TrackerStatus ts);

TrackerStatus resetTracker(TrackerData * td,long timeout);
TrackerStatus sendTrackerCMD(TrackerData * td,char * cmd,long timeout);
TrackerStatus getTrackerResponse(TrackerData * td,long timeout);

TrackerStatus getTrackerVerStr(TrackerData * td,char * str,long timeout);
TrackerStatus getTrackerVerRev(TrackerData * td,
                               long * hver,long * hrev,
                               long * fver,long * frev,
                               long timeout);

#endif

/* tracker.h */
/* trw.h */
/* Created 12/16/94 */
/* Ken Hawkins */

#ifndef TRW_H
#define TRW_H

TrackerStatus readTracker8(TrackerData * td,long addr,long * val);

TrackerStatus writeTracker8(TrackerData * td,long addr,long val);

TrackerStatus readTracker16(TrackerData * td,long addr,long * val);

TrackerStatus writeTracker16(TrackerData * td,long addr,long val);


#endif

/* trw.h */

/* tcal.h */
/* Created 12/16/94 */
/* Ken Hawkins */

#ifndef TCAL_H
#define TCAL_H

// Misc. var offsets               // Var size:
#define DATA_MODE_OFFSET      (0x04) // byte
#define MOUSE_MODE_OFFSET     (0x05) // byte
#define SEND_MODE_OFFSET      (0x06) // byte
#define DATA_FORMAT_OFFSET    (0x07) // byte

#define MOUSE_X_SENSI_OFFSET  (0x42) // byte
#define MOUSE_X_THRESH_OFFSET (0x43) // byte

#define MOUSE_Y_SENSI_OFFSET  (0x46) // byte
#define MOUSE_Y_THRESH_OFFSET (0x47) // byte

typedef enum {TA_X,TA_Y,TA_Z,TA_P,TA_R} TrackerAxis;

void setBaseAddr(unsigned long addr);

unsigned long getBaseAddr(void);

TrackerStatus readTrackerMiscVar(TrackerData * td,long offset,long * val);

TrackerStatus readTrackerGain(TrackerData * td,TrackerAxis ta);

TrackerStatus writeTrackerGain(TrackerData * td,TrackerAxis ta);

TrackerStatus readTrackerOffset(TrackerData * td,TrackerAxis ta);

TrackerStatus writeTrackerOffset(TrackerData * td,TrackerAxis ta);

TrackerStatus readTrackerOffsets(TrackerData * td);

TrackerStatus writeTrackerOffsets(TrackerData * td);

TrackerStatus readTrackerGains(TrackerData * td);

TrackerStatus writeTrackerGains(TrackerData * td);

TrackerStatus readTrackerMagFilts(TrackerData * td);

TrackerStatus writeTrackerMagFilts(TrackerData * td);

TrackerStatus readTrackerGrvFilts(TrackerData * td);

TrackerStatus writeTrackerGrvFilts(TrackerData * td);

TrackerStatus readTrackerFilters(TrackerData * td);

TrackerStatus writeTrackerFilters(TrackerData * td);

TrackerStatus readTrackerAllVars(TrackerData * td);

TrackerStatus writeTrackerAllVars(TrackerData * td);

char * getOffsetsStr(TrackerData * td);

char * getGainsStr(TrackerData * td);

char * getFiltersStr(TrackerData * td);

char * getAllVarsStr(TrackerData * td);

#endif

/* tcal.h */

/* spacerat.h */
/* Created 1/6/95 at CES in LAS VEGAS */
/* Ken Hawkins */

#ifndef SPACERAT_H
#define SPACERAT_H

void spacerat(TrackerData * td);

#endif

/* spacerat.h */

/* vtmode.h */
/* Created 3/21/95 */
/* Ken Hawkins */

#ifndef VTMODE_H
#define VTMODE_H

void vtmode(char * msg,int argc,char ** argv);

#endif

/* vtmode.h */

