/* clip.c */
/* Created 1/15/95 */
/* Copyright 1998 Ken Hawkins */
/* All Rights Reserved */

#include "imath1.h"
#include "vector1.h"
#include "clip.h"

#define RIGHT_HANDED_COORDS

static stereoSMC * smcp;

//*** Stereo separation, magnification, and clipping setup ***

void setStereoSMC(stereoSMC * smc) {
  smcp = smc;
} // setStereoSMC

// *** Projection ***

void project3mono(vec2 * pm,vec3 * p) {
  pm->x = smcp->center.x - MULDIV(p->x,smcp->vmag.x,p->z);
  pm->y = smcp->center.y + MULDIV(p->y,smcp->vmag.y,p->z);
} // project3mono

void project3stereo(vec2 * ps,vec3 * p) {
  ps->x = smcp->center.x - MULDIV(p->x+smcp->msep,smcp->vmag.x,p->z);
  ps->y = smcp->center.y + MULDIV(p->y,smcp->vmag.y,p->z);
} // project3stereo

long clipPoint2(vec2 * p) {
  return (p->x >= smcp->clip.min.x && p->x <= smcp->clip.max.x &&
	  p->y >= smcp->clip.min.y && p->y <= smcp->clip.max.y);
} // clipPoint2

long clipPoint3(vec3 * p) {
  return (p->z >= smcp->clip.min.z && p->z <= smcp->clip.max.z);
} // clipPoint3

/*** 2D Line Clipper ***/

#define TOPCODE    (1<<0)
#define BOTTOMCODE (1<<1)
#define LEFTCODE   (1<<2)
#define RIGHTCODE  (1<<3)

#define CODEPOINT(code,xc,yc) code=0;\
  if ((yc) > smcp->clip.max.y) code |= BOTTOMCODE;\
  else if ((yc) < smcp->clip.min.y) code |= TOPCODE;\
  if ((xc) > smcp->clip.max.x) code |= RIGHTCODE;\
  else if ((xc) < smcp->clip.min.x) code |= LEFTCODE;

#define CLIP(m,n,a,b,c,d,edge) {\
  (m) = (a) + MULDIV((b) - (a),(edge) - (c),(d) - (c));\
  (n) = edge;}

#define RECODE(code,x,y,xn,yn) {(x)=(xn);(y)=(yn);CODEPOINT(code,(x),(y));}

#define X0 l[0].x
#define Y0 l[0].y
#define X1 l[1].x
#define Y1 l[1].y

long clipLine2(vec2 * l) {
long x,y;
long c,c0,c1;
  CODEPOINT(c0,X0,Y0);
  CODEPOINT(c1,X1,Y1);
  while (1) {
	if (c0) {
	  if (c0 & c1) return 0;
	  c = c0;
	} else if (c1) c = c1;
	else return 1;
	if (TOPCODE    & c) CLIP(x,y,X0,X1,Y0,Y1,smcp->clip.min.y) else
	if (BOTTOMCODE & c) CLIP(x,y,X0,X1,Y0,Y1,smcp->clip.max.y) else
	if (RIGHTCODE  & c) CLIP(y,x,Y0,Y1,X0,X1,smcp->clip.max.x) else
	if (LEFTCODE   & c) CLIP(y,x,Y0,Y1,X0,X1,smcp->clip.min.x)
	if (c == c0) RECODE(c0,X0,Y0,x,y)
	else         RECODE(c1,X1,Y1,x,y)
  } // while
} // clipLine2

#ifdef RIGHT_HANDED_COORDS

long clipLine3(vec3 * l) {
vec3 * p0,* p1;
long delta1,delta2;
  if (l[0].z > l[1].z) {
    p0 = &l[0];
    p1 = &l[1];
  } else {
    p0 = &l[1];
    p1 = &l[0];
  } // if
  if (p0->z > smcp->clip.min.z) {
    if (p1->z > smcp->clip.min.z) return 0;
    delta1 = smcp->clip.min.z - p0->z;
    delta2 = p1->z - p0->z;
    p0->x += MULDIV(p1->x - p0->x,delta1,delta2);
    p0->y += MULDIV(p1->y - p0->y,delta1,delta2);
    p0->z = smcp->clip.min.z;
  } // if
  if (p1->z < smcp->clip.max.z) {
    if (p0->z < smcp->clip.max.z) return 0;
    delta1 = smcp->clip.max.z - p1->z;
    delta2 = p0->z - p1->z;
    p1->x += MULDIV(p0->x - p1->x,delta1,delta2);
    p1->y += MULDIV(p0->y - p1->y,delta1,delta2);
    p1->z = smcp->clip.min.z;
  } // if
  return 1;
} // clipLine3

#else // Left-handed coordinate system

long clipLine3(vec3 * l) {
vec3 * p0,* p1;
long delta1,delta2;
  if (l[0].z < l[1].z) {
    p0 = &l[0];
    p1 = &l[1];
  } else {
    p0 = &l[1];
    p1 = &l[0];
  } // if
  if (p0->z < smcp->clip.min.z) {
    if (p1->z < smcp->clip.min.z) return 0;
    delta1 = smcp->clip.min.z - p0->z;
    delta2 = p1->z - p0->z;
    p0->x += MULDIV(p1->x - p0->x,delta1,delta2);
    p0->y += MULDIV(p1->y - p0->y,delta1,delta2);
    p0->z = smcp->clip.min.z;
  } // if
  if (p1->z > smcp->clip.max.z) {
    if (p0->z > smcp->clip.max.z) return 0;
    delta1 = smcp->clip.max.z - p1->z;
    delta2 = p0->z - p1->z;
    p1->x += MULDIV(p0->x - p1->x,delta1,delta2);
    p1->y += MULDIV(p0->y - p1->y,delta1,delta2);
    p1->z = smcp->clip.min.z;
  } // if
  return 1;
} // clipLine3

#endif

// *** Projection and clipping ***

// *** Mono ***

long projClipPointMono(vec2 * out,vec3 * in) {
  if (!clipPoint3(in)) return 0;
  project3mono(out,in);
  return clipPoint2(out);
} // projClipPointMono

long projClipLineMono(vec2 * out,vec3 * in) {
  if (!clipLine3(in)) return 0;
  project3mono(&out[0],&in[0]);
  project3mono(&out[1],&in[1]);
  return clipLine2(out);
} // projClipLineMono

// ** Stereo ***

long projClipPointStereo(vec2 * out,vec3 * in) {
  if (!clipPoint3(in)) return 0;
  project3stereo(out,in);
  return clipPoint2(out);
} // projClipPointStereo

long projClipLineStereo(vec2 * out,vec3 * in) {
  if (!clipLine3(in)) return 0;
  project3stereo(&out[0],&in[0]);
  project3stereo(&out[1],&in[1]);
  return clipLine2(out);
} // projClipLineStereo

/* clip.c */
