/* Convert.c */
/* Converts raw tracker data to orientation information */
/* Created 11/17/94. New method for atan2 with no sqrt 1/30/95. */
/* Ken Hawkins */

/*
  The coordinate system used here is y up/down, z in/out, x right/left.
  (Right-handed).
*/

#include <stdio.h>
#include "imath1.h"
#include "vector1.h"
#include "convert.h"


#define USE_FILTERS

#ifdef USE_FILTERS
#include "filter1.h"
static long madeFilters=0;
static filter f1,f2,fx,fy,fz;
#endif

void rawToEulerInt(long rawMagneticX,long rawMagneticY,long rawMagneticZ,
                   long rawGravimetricX,long rawGravimetricY,
                   long * eulerYaw,long * eulerPitch,long * eulerRoll) {
vec3 rotMagnetic;
vec2 linGravimetric;
vec2 normRotMag;
sinCos pitch,roll;

#ifdef USE_FILTERS
  if (!madeFilters) {
    if (!initFIR(&fx,3)) return;
    if (!initFIR(&fy,3)) return;
    if (!initFIR(&fz,3)) return;
    if (!initFIR(&f1,6)) return;
    if (!initFIR(&f2,6)) return;
    madeFilters = 1;
  } // if

  rawMagneticX = filterFIR(&fx,rawMagneticX);
  rawMagneticY = filterFIR(&fy,rawMagneticY);
  rawMagneticZ = filterFIR(&fz,rawMagneticZ);

  linGravimetric.x = filterFIR(&f1,MUL(rawGravimetricX,RAD180DEG));
  linGravimetric.y = filterFIR(&f2,MUL(rawGravimetricY,RAD180DEG));
#else
  linGravimetric.x = MUL(rawGravimetricX,RAD180DEG);
  linGravimetric.y = MUL(rawGravimetricY,RAD180DEG);
#endif

  sincos(&pitch,linGravimetric.x);
  sincos(&roll,linGravimetric.y);

// Inversely transform the magnetic vector back into definition space.
// This allows an easy computation of yaw by looking at the projection
// of the magnetic vector in the x-z plane, which is the plane of the
// earth.

// Perform a rotation about x: Pitch
  rotMagnetic.y = MUL(pitch.cos,rawMagneticY) - MUL(pitch.sin,rawMagneticZ);
  rotMagnetic.z = MUL(pitch.sin,rawMagneticY) + MUL(pitch.cos,rawMagneticZ);

// Perform a rotation about z: Roll. We only need x, so y isn't transformed.
  rotMagnetic.x = MUL(roll.cos,rawMagneticX) - MUL(roll.sin,rotMagnetic.y);

  *eulerYaw   = TODEGREES(getatan2(rotMagnetic.z,rotMagnetic.x));
  *eulerPitch = TODEGREES(linGravimetric.x);
  *eulerRoll  = TODEGREES(linGravimetric.y);

} // rawToEulerInt

/* convert.c */
