/* dramgfx.c */
/* Created 2/03/95 */
/* Copyright 1998 Ken Hawkins */
/* All Rights Reserved */

#include <conio.h>

#include "imath1.h"
#include "vector1.h"

#define SC_INDEX 0x03C4

// Video memory page 0
#define VMPAGE0 0xa0000

static char * vmpage[4] = {VMPAGE0,VMPAGE0+80,VMPAGE0+(1<<15),VMPAGE0+(1<<15)+80};
static unsigned long yOffset[400];

void d_computeOffsets(long stereo) {
long i;
  for (i=0; i < 400; i++) yOffset[i] = i*320;
} // d_computeOffsets

void d_clearPage400(char * pageAddr,long color) {
register unsigned long * p = (unsigned long *)pageAddr;
long i;
  color &= 0xff;
  color |= color<<8;
  color |= color<<16;
  for (i=0; i < 320*400/4/4; i++) {
	*p++ = color;
	*p++ = color;
	*p++ = color;
	*p++ = color;
  } // for
} // d_clearPage400

void d_copyPage400(long destPage,char * srcPage) {
unsigned long * destAddr;
long i,j;
  outp(SC_INDEX,2); // Set up (point to) plane mask register
  for (j=0; j < 4; j++) {
	outp(SC_INDEX+1,1<<j); // Point to specific plane
	destAddr = (unsigned long *)vmpage[destPage];
	for (i=0; i < 320*400; i += 16) {
	  long ij = i+j;
	  *destAddr++ = srcPage[ij] | (srcPage[ij+4]<<8) | (srcPage[ij+8]<<16) | (srcPage[ij+12]<<24);
	} // for
  } // for
} // d_copyPage400

static void copyPage200i(long destPage,char * srcPage) {
unsigned long * destAddr;
long i,j,k;
  outp(SC_INDEX,2); // Set up (point to) plane mask register
  for (j=0; j < 4; j++) {
	outp(SC_INDEX+1,1<<j); // Point to specific plane
	destAddr = (unsigned long *)vmpage[destPage];
	for (k=0; k < 320*200; k += 320) {
	  long jk = j+k;
	  for (i=0; i < 320; i += 16) {
		long ijk = i+jk;
		*destAddr++ = srcPage[ijk] | (srcPage[ijk+4]<<8) | (srcPage[ijk+8]<<16) | (srcPage[ijk+12]<<24);
	  } // for
	  destAddr += 20; // 80/4: skip the next scanline.
	} // for
  } // for
} // copyPage200i

void d_copyPagesStereo400(long destPage,char * srcPageL,char * srcPageR) {
  copyPage200i(destPage,  srcPageL);
  copyPage200i(destPage+1,srcPageR);
} // d_copyPagesStereo400

void d_setPixel(char * pageAddr,vec2 * p,long color) {
  pageAddr[p->y*320+p->x] = color;
} // d_setPixel

void d_drawLine(char * pageAddr,vec2 * l,long color) {
long x1 = l[0].x;
long y1 = l[0].y;
long x2 = l[1].x;
long y2 = l[1].y;
long d,x,y,ax,ay,sx,sy,dx,dy;
register char * addr = pageAddr;
  dx = x2-x1; ax = abs(dx)<<1; sx = sgn(dx);
  dy = y2-y1; ay = abs(dy)<<1; sy = sgn(dy);
  x = x1;
  y = y1;
  if (ax > ay) { /* x dominant */
	d = ay - (ax>>1);
	while (1) {
	  addr[yOffset[y]+x] = color;
//      addr[y*320+x] = color;
	  if (x == x2) return;
	  if (d >= 0) {
		y += sy; d -= ax;
	  } // if
	  x += sx; d += ay;
	} // while
  } else {       /* y dominant */
	d = ax - (ay>>1);
	while (1) {
	  addr[yOffset[y]+x] = color;
//      addr[y*320+x] = color;
	  if (y == y2) return;
	  if (d >= 0) {
		x += sx; d -= ay;
	  } // if
	  y += sy; d += ax;
	} // while
  } // if
} // d_drawLine

void d_fillRect(char * pageAddr,vec2 * extent,long color) {
long x1 = extent[0].x;
long y1 = extent[0].y;
long x2 = extent[1].x;
long y2 = extent[1].y;
register char * addr = pageAddr;
long x,y,temp;
  if (x1 > x2) {temp = x1; x1 = x2; x2 = temp;}
  if (y1 > y2) {temp = y1; x1 = y2; y2 = temp;}
  for (x=x1; x <= x2; x++) {
	for (y=y1; y <= y2; y++) {
	  addr[y*320+x] = color;
	} // for
  } // for
} // d_fillRect

/* dramgfx.c */
