/* filter.c */
/* Created 11/22/94 */
/* Ken Hawkins */

#include <stdlib.h>
#include "imath1.h"
#include "filter.h"

#define LEN_HISTORY 128

#if 0
static float fweights[LEN_HISTORY];
#endif

void initWeights(filter * f) {
#if 0
float sum;
long i;

// Generate weights.
  for (i=0; i < f->len; i++) {
    fweights[i] = 1.0;
  } /* for */

// Summate.
  sum = 0.;
  for (i=0; i < f->len; i++) {
    sum += fweights[i];
  } /* for */

// Normalize and convert to fixed point.
  for (i=0; i < f->len; i++) {
    f->weights[i] = TOFIXED(fweights[i]/sum);
  } /* for */

#endif

} /* initWeights */

void initHistory(filter * f) {
long i;
  for (i=0; i < f->len; i++) {
    f->history[i] = TOFIXED(1.0);
  } /* for */
  f->hCurrent = f->history;
  f->hEnd     = &f->history[f->len-1];
  f->hRestart = &f->history[-1];
} /* initHistory */

long initFIR(filter * f,long len) {
  f->history = calloc(len,sizeof(coord));
  if (!f->history) return 0;
  f->weights = calloc(len,sizeof(coord));
  if (!f->weights) return 0;
  f->len = len;
  initWeights(f);
  initHistory(f);
  return 1;
} // initFIR

void freeFIR(filter * f) {
  if (f->history) {
    free(f->history);
    f->history = 0;
  } // if
  if (f->weights) {
    free(f->weights);
    f->weights = 0;
  } // if
} // freeFIR

coord filterFIR(filter * f,coord newval) {
coord * currp,* last;
coord * weightp;
coord sum;

  currp   = f->hCurrent;
  last    = f->hCurrent;
  weightp = f->weights;

  *(f->hCurrent)++ = newval;
  if (f->hCurrent == &f->history[f->len]) f->hCurrent = f->history;

// Compute FIR filter
  sum = 0;
  while (1) {
//    sum += MUL((*currp--),(*weightp++));
    sum += *currp--;
    if (currp == f->hRestart) currp = f->hEnd;
    if (currp == last) break;
  } /* while */

  return sum/f->len;

} /* filterFIR */

long filterInterp(ifilter * f,long rawNum) {
long interp;

  interp = (f->filteredNum - rawNum)*f->filterRate + f->remainder;
  f->remainder = interp & 0xffff;
  f->filteredNum = rawNum + (interp>>16);
  return f->filteredNum;

} // filterInterp

/* filter.c */
