/* gfx.c */
/* Created 12/12/94 John Schwab */
/* Modified 12/14/94 Ken Hawkins */

#include <stdio.h>
#include <graph.h>
#include "gfx.h"

static long gfxOpened = 0;

GraphicsStatus initGraphics( gfxConfig *gc, GraphicsMode gm) {
  struct videoconfig VC;
  gfxOpened = 1;
  switch( gm ){
	case GM_640x480x16:
	if( _setvideomode( _VRES16COLOR ) == 0) return GS_NA;
	break;
	case GM_320x200x256:
	if( _setvideomode( _MRES256COLOR ) == 0) return GS_NA;
	break;
	case GM_640x400x256:
	if( _setvideomode( _URES256COLOR ) == 0) return GS_NA;
	break;
	case GM_640x480x256:
	if( _setvideomode( _VRES256COLOR ) == 0) return GS_NA;
	break;
	case GM_800x600x256:
	if( _setvideomode( _SVRES256COLOR ) == 0) return GS_NA;
	break;
	case GM_1024x768x256:
	if( _setvideomode( _XRES256COLOR ) == 0) return GS_NA;
	break;
  }
  _getvideoconfig( &VC );
  gc->width = VC.numxpixels;
  gc->height = VC.numypixels;
  gc->numColors = VC.numcolors;
  return GS_OK;
} // initGraphics

void CloseGraphics(void){
  if( gfxOpened ){
	gfxOpened = 0;
	_setvideomode( _DEFAULTMODE ); //restore the graphics mode
  }
}  //CloseGraphics

void writePixel( int x, int y){
  _setpixel( x, y);
}

void writePixelColor( int x, int y, BYTE color){
  _setcolor( color );
  _setpixel( x, y);
}

void writeLine( int x1, int y1, int x2, int y2){
  _moveto( x1, y1 );
  _lineto( x2, y2 );
}

void writeLineColor( int x1, int y1, int x2, int y2, BYTE color){
  _setcolor( color );
  _moveto( x1, y1 );
  _lineto( x2, y2 );
}

void writeCircle( int x, int y, int radius){
  int x1, y1, x2, y2;
  x1 = x - radius;
  y1 = y - radius;
  x2 = x + radius;
  y2 = y + radius;
  _ellipse( _GBORDER, x1, y1, x2, y1);
}

void writeCircleColor( int x, int y, int radius, BYTE color){
  int x1, y1, x2, y2;
  _setcolor( color );
  x1 = x - radius;
  y1 = y - radius;
  x2 = x + radius;
  y2 = y + radius;
  _ellipse( _GBORDER, x1, y1, x2, y2);
}

void writeFillCircleColor( int x, int y, int radius, BYTE color){
  int x1, y1, x2, y2;
  _setcolor( color );
  x1 = x - radius;
  y1 = y - radius;
  x2 = x + radius;
  y2 = y + radius;
  _ellipse( _GFILLINTERIOR, x1, y1, x2, y2);
}

void writeBox( int x1, int y1, int x2, int y2){
  _rectangle( _GBORDER, x1, y1, x2, y2);
}

void writeBoxColor( int x1, int y1, int x2, int y2, BYTE color){
  _setcolor( color );
  _rectangle( _GBORDER, x1, y1, x2, y2);
}

void writeFillBox( int x1, int y1, int x2, int y2){
  _rectangle( _GFILLINTERIOR, x1, y1, x2, y2);
}

void writeFillBoxColor( int x1, int y1, int x2, int y2, BYTE color){
  _setcolor( color );
  _rectangle( _GFILLINTERIOR, x1, y1, x2, y2);
}

void writeText( int x, int y, BYTE *string){
//      _setcharsize( width, width * 3 / 2 );
//      _settextalign( x, y );
  _grtext( x, y, string );
}

void writeTextColor( int x, int y, BYTE *string, BYTE color){
  _setcolor( color );
//      _setcharsize( width, width * 3 / 2 );
//      _settextalign( x, y );
  _grtext( x, y, string );

}

void outText(long col,long row,char * s,long color) {
  _settextcolor(color);
  _settextposition(row,col);
  _outtext(s);
} // outText

void drawWindow(int x1, int y1, int width, int height, int thick){
  writeFillBoxColor( x1 + thick, y1 + thick, x1 + width - thick, y1 + height - thick, 7);
  writeFillBoxColor( x1, y1, x1 + width, y1 + thick - 1, 15);
  writeFillBoxColor( x1, y1, x1 + thick - 1, y1 + height, 15);
  writeFillBoxColor( x1, y1 + height, x1 + width, y1 + height - thick + 1, 0);
  writeFillBoxColor( x1 + width, y1, x1 + width - thick + 1, y1 + height, 0);
}

/* gfx.c */

