/* gridemo.c */
/* Created 1/19/95 */
/* Ken Hawkins */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <ctype.h>
#include "imath1.h"
#include "vector1.h"
#include "clip1.h"
#include "vgfx1.h"
#include "dramgfx1.h"
#include "tracker1.h"
#include "timer1.h"
#include "serial1.h"
#include "vfont1.h"

#include "grid.h"
#include "gridemo.h"

static TrackerStatus ts;

static stereoSMC smcl,smcr;
static coord zoom,msep,sep;
static vfont vf;
static vec3 vcenter;
static long background = 0;
static float frameRate;

static void displayAngles(long page,matrix34 * m,vec3 * center,
                          coord yaw,coord pitch,coord roll) {
long color;
char s[128];
  if (background) color = DCOLOR(0,0,0);
  else color = DCOLOR(0xff,0xff,0xff);
  sprintf(s,"Y %6.2f P %6.2f R %6.2f",
    TOFLOAT(yaw),TOFLOAT(pitch),TOFLOAT(roll));
  drawVString(page,&vf,s,m,center,VF_ALIGN_CENTER,color);

  {vec3 c = *center;
   c.y -= vf.interLineSpace+vf.height+1;
  sprintf(s,"%2.f fps",frameRate);
  drawVString(page,&vf,s,m,&c,VF_ALIGN_CENTER,color);
  }

} // displayAngles

static void initSMC(long stereo,coord msep,coord sep,coord zoom) {

  smcl.vmag.x = zoom;
  if (stereo) {
    smcl.msep       = msep;
    smcl.center.x   = 159 - sep;
    smcl.vmag.y     = MUL(zoom,TOFIXED(10./11.));
    smcl.center.y   =  99;
    smcl.clip.max.y = 199;
  } else {
    smcl.msep       = 0;
    smcl.center.x   = 159;
    smcl.vmag.y     = MUL(zoom,TOFIXED(10./11.*2.));
    smcl.center.y   = 199;
    smcl.clip.max.y = 399;
  } // if
  smcl.clip.min.x = 0;
  smcl.clip.min.y = 0;
  smcl.clip.max.x = 319;
  smcl.clip.min.z = TOFIXED(-.25);
  smcl.clip.max.z = TOFIXED(-2500);
  smcr = smcl;
  smcr.msep = -smcr.msep;
  smcr.center.x = 159 + sep;

  computeOffsets(stereo);   // vgfx scanline offsets for mono/stereo
  d_computeOffsets(stereo); // dramgfx scanline offsets

// Center for vector font
  vcenter.x = (smcl.clip.max.x - smcl.clip.min.x + 1)/2;
  vcenter.y = smcl.clip.max.y - vf.interLineSpace*4;

} // initSMC

static void defaultView(void) {
  msep = 2729;
  sep  = 11;
  zoom = TOFIXED(.125/16.+.0005);
} // defaultView

static void getViewMatrix(TrackerData * td,matrix34 * m,coord size) {
matrix34 x,y,z,t;
static vec3 axisX = {ONE,0,0};
static vec3 axisY = {0,ONE,0};
static vec3 axisZ = {0,0,ONE};
static long angle = 0;

  matrix33axis(&x.m,&axisX,angle);
  matrix33axis(&y.m,&axisY,angle);

  td->euler.x = td->euler.y = angle;

  matrix33axis(&z.m,&axisZ,angle += 200);

  td->euler.z = angle;

  matrixMul33(&t.m,&x.m,&y.m);
  matrixMul33(&m->m,&t.m,&z.m);

// For a static test image:
//  matrix33unitize(&m->m);

  SETVEC(m->w,0,MUL(size,TOFIXED(1./2.5)),0);

} // getViewMatrix

static void getTrackerViewMatrix(TrackerData * td,matrix34 * m,long trackerMode,coord size) {
matrix33 x,y,z,t;
static vec3 axisX = {ONE,0,0};
static vec3 axisY = {0,ONE,0};
static vec3 axisZ = {0,0,ONE};

  if (trackerMode == GRID_MODE_MICRO_EULER) {
    ts = readTrackerEuler(td,timerSecs(1));
    if (ts != TS_OK) {
      printf("\n%s.\n",trackerInfo(ts));
      flushSer(td->sd);
    } // if
  } else {
    ts = readTracker(td,timerSecs(1));
    if (ts != TS_OK) {
      printf("\n%s.\n",trackerInfo(ts));
      flushSer(td->sd);
    } // if
  } // if
  requestTrackerData(td);

  matrix33axis(&x,&axisX,TORADIANS(td->euler.x));
  matrix33axis(&y,&axisY,TORADIANS(td->euler.y));
  matrix33axis(&z,&axisZ,TORADIANS(td->euler.z));

  matrixMul33(&t,&z,&x);
  matrixMul33(&m->m,&t,&y);

  SETVEC(m->w,0,MUL(size,TOFIXED(1./2.5)),0);

} // getTrackerViewMatrix

void gridDemo(TrackerData * td,long mode,long trackerMode,
              long lines,coord size,long useVSync) {
matrix34 m,vm;
char * dram;
long frames,t1,t2,dt;
long bcolor     = 0;
long useDram    = 0;
long stereo     = 0;
long dpage      = 0;
long vpage      = 1;
long showAngles = 1;
long quit       = 0;

  if (!loadVFont("simple.vft",&vf)) {
    printf("Couldn't load font: simple.vft.\n");
    exit(0);
  } // if

  scaleVFont(&vf,TOFIXED(1.5),TOFIXED(6.),TOFIXED(1.));

  dram = calloc(320,400);
  if (!dram) {
    printf("Not enough memory for drawing buffer.\n");
    exit(0);
  } // if

  if (!initGrid(lines,size)) {
    printf("Not enough memory for grid.\n");
    exit(0);
  } // if

  matrix33unitize(&vm.m);

  initXmode();

  defaultView();
  initSMC(stereo,msep,sep,zoom);

  t1 = getTime();
  frames = 0;

  quit = 0;
  while (!quit) {

    if (kbhit()) {
      char ch = getch();
      ch = toupper(ch);
      switch (ch) {
        case '\\': defaultView(); break;
        case 'A': showAngles ^= 1; break;
        case '7': sep -= 1; break;
        case '8': sep += 1; break;
        case '9': msep -= TOFIXED(.1); break;
        case '0': msep += TOFIXED(.1); break;
        case '-': zoom -= TOFIXED(.0001); break;
        case '=': zoom += TOFIXED(.0001); break;
        case 'S': stereo ^= 1; break;
        case 'D': useDram ^= 1; break;
        case 'B': background ^= 1;
          if (background) bcolor = DAC(2,2,1); else bcolor = 0; break;
        case 'V': useVSync ^= 1; break;
        case 0x1b: // ESC
        case 'Q': quit = 1; break;
      } // switch
      initSMC(stereo,msep,sep,zoom);
    } // if

    if (mode == 0) {
      getViewMatrix(td,&m,size);
    } else {
      getTrackerViewMatrix(td,&m,trackerMode,size);
    } // if

    _disable();
    setViewPageAddr(vpage<<15);
    _enable();

    if (useVSync) waitVS();

    if (useDram) {
      d_clearpage400(dram,bcolor);
    } else {
      clearpage400(dpage,bcolor);
    } // if

    if (useDram) {
      if (stereo) {
        setStereoSMC(&smcl);
        d_drawGrid(dram,&m);
        setStereoSMC(&smcr);
        d_drawGrid(&dram[320*200],&m);
        d_copyPagesStereo400(dpage,dram,&dram[320*200]);
      } else {
        setStereoSMC(&smcl);
        d_drawGrid(dram,&m);
        d_copyPage400(dpage,dram);
      } // if
    } else {
      if (stereo) {
        setStereoSMC(&smcl);
        drawGrid(dpage,&m);
        if (showAngles) displayAngles(dpage,0,&vcenter,td->euler.y,td->euler.x,td->euler.z);
        setStereoSMC(&smcr);
        drawGrid(dpage+1,&m);
        if (showAngles) displayAngles(dpage+1,0,&vcenter,td->euler.y,td->euler.x,td->euler.z);
      } else {
        setStereoSMC(&smcl);
        drawGrid(dpage,&m);
        if (showAngles) displayAngles(dpage,0,&vcenter,td->euler.y,td->euler.x,td->euler.z);
      } // if
    } // if

    vpage += 1;
    if (vpage == 2) vpage = 0;
    dpage += 2;
    if (dpage == 4) dpage = 0;

    frames++;

    if (frames == 70) {
      t2 = getTime();
      frameRate = computeFrameRate(frames,t1,t2);
      frames = 0;
      t1 = t2;
    } // if

  } // while

  deInitXmode();

  deInitGrid();

  free(dram);

  freeVFontMembers(&vf);

} // gridDemo

