/* test.c */
/* Created 1/15/95 */
/* Ken Hawkins */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include "imath1.h"
#include "vector1.h"
#include "tracker1.h"
#include "gridemo.h"

static TrackerData td;
static TrackerStatus ts;

// Global command-line argument count and values.
static long ac;
static char ** av;

long checkParm(char * parm) {
long i;
  for(i = 1; i < ac; i++) if(!strcmpi(av[i],parm)) return i;
  return 0; // parm not found
} // checkParm

void main(int argc,char ** argv) {
TrackerPort trackerPort;
long lines;
long usevsync;
long testmode;
coord size;
long baud;
long i;

// Setup global arg vals for checkParm().
  ac = argc;
  av = argv;

  if (checkParm("-?") || checkParm("/?") || checkParm("?") ||
    checkParm("-h") || checkParm("/h") || checkParm("h")) {
    printf("\n\n\nUsage:\n");
    printf("  grid [-t] [-lines #] [-size #] [-novsync]\n");
    printf("    -t       : Run grid in test mode (doesn't use tracker).\n");
    printf("    -lines # : Number of lines per cube face (2-256).\n");
    printf("    -size #  : Cube size (1-2000) [Affects stereo 3D].\n");
    printf("    -novsync : Don't use video sync (run at max speed).\n");
    printf("    -baud #  : Baud rate: 1200, 2400, 4800, 9600, or 19200.\n");
    printf("Example:\n  grid -novsync -lines 16 -size 10 -baud 9600\n");
    printf("The default values are: lines = 12, size = 20, baud = 19200.\n");
    printf("The following keys are used while grid is running:\n");
    printf("    s        : Toggle Stereoscopic 3D on and off.\n");
    printf("    -,=      : Zoom out, zoom in view.\n");
    printf("    7,8      : Decrease, increase stereo separation.\n");
    printf("    9,0      : Decrease, increase stereo interocular angle.\n");
    printf("    \\        : Reset view to default.\n");
    printf("    a        : Toggle Angle and frame rate display on and off.\n");
    printf("    b        : Toggle Background color.\n");
    printf("    d        : Toggle DRAM offscreen rendering on and off.\n");
    printf("    v        : Toggle video sync on and off.\n");
    printf("    q,ESC    : Quit.\n");
    exit(0);
  } // if

  testmode = checkParm("-t") != 0;
  usevsync = checkParm("-novsync") == 0;

  if (i=checkParm("-lines")) {
    if (!sscanf(av[i+1],"%ld",&lines)) {
      printf("A number must be specified after the -lines argument.\n");
      printf("For example: -lines 12\n");
      exit(1);
    } // if
    CLAMPMINMAX(lines,2,256);
  } else {
    lines = 12;
  } // if

  if (i=checkParm("-size")) {
    if (!sscanf(av[i+1],"%ld",&size)) {
      printf("A number must be specified after the -size argument.\n");
      printf("For example: -size 20\n");
      exit(1);
    } // if
    CLAMPMINMAX(size,1,2000);
  } else {
    size = 20;
  } // if

  if (i=checkParm("-baud")) {
    if (!sscanf(av[i+1],"%ld",&baud)) {
      printf("A valid baud rate must be specified after the -baud argument.\n");
      printf("For example: -baud 19200\n");
      exit(1);
    } // if
    if (baud != 1200 && baud != 2400 && baud != 4800
     && baud != 9600 && baud != 19200) {
      printf("Baud must be 1200, 2400, 4800, 9600, or 19200.\n");
      exit(1);
    } // if
  } else {
    baud = 19200;
  } // if

  if (testmode) {
    gridDemo(&td,0,0,lines,SHL(size),usevsync);
  } else {

    ts = initTrackerSearch(&td,&trackerPort,baud,timerSecs(2));
    printf("%s.\n",trackerInfo(ts));
    if (ts != TS_OK) {
      closeTracker(&td);
      exit(0);
    } // if

    ts = sendTrackerCMD(&td,"!M1,P,B\r",timerSecs(2));
    if (ts != TS_OK) {
      printf("%s.\n",trackerInfo(ts));
      closeTracker(&td);
      exit(0);
    } // if

    requestTrackerData(&td);

    gridDemo(&td,1,GRID_MODE_COOKED,lines,SHL(size),usevsync);

    closeTracker(&td);
  } // if
} // main
