/* imath.c */
/* Created 12/14/94. New method for atan2 1/30/95 */
/* Copyright 1998 Ken Hawkins */
/* All Rights Reserved */

#include <math.h>
#include "imath.h"

// Double precision fixed point (28 bit) 1.0
#define D_ONE (1L<<(FBITS*2L))

//#define MAKE_TABLES
//#define PRINT_TABLES
//#define USE_FLOAT_FUNCS

#define SIN_TABLE_BITS 8L

#define SIN_TABLE_SIZE ((1L<<SIN_TABLE_BITS)+1L)
#define SIN_TABLE_SHIFT (FBITS - SIN_TABLE_BITS)
#define SIN_INTERP_MASK ((1L<<SIN_TABLE_SHIFT)-1L)

#define ATAN_TABLE_BITS 8L

#define ATAN_TABLE_SIZE ((1L<<ATAN_TABLE_BITS)+1L)
#define ATAN_TABLE_SHIFT (FBITS - ATAN_TABLE_BITS)
#define ATAN_INTERP_MASK ((1L<<ATAN_TABLE_SHIFT)-1L)

#if defined(__WATCOMC__)

// The '6' below is from FBITS-ATAN_TABLE_BITS (14-8=6)
// a.k.a. ATAN_TABLE_SHIFT.
long mula(long a,long b);
#pragma aux mula =       \
	"imul   edx"         \
	"shrd   eax,edx,6"   \
	"adc    eax,0"       \
	parm [eax] [edx]     \
	modify [eax edx]     \
	value [eax];

#elif defined(__BORLANDC__)

// Invoke "compile via assembly" (Same as command line -B option)
// This is required as the bcc assembler can't handle 32 bit registers/instructions
#pragma inline

// In Borland C, ax and dx are return value registers (for 32 bit return values)

coord mula(long a,long b) {
long c;
  asm {
	.386
	mov  eax,DWORD PTR a
    imul DWORD PTR b
    shrd eax,edx,6
    adc  eax,0
    mov  DWORD PTR c,eax
  } // asm
  return c;
} // mul64

coord mul64(long a,long b) {
long c;
  asm {
	.386
	mov  eax,DWORD PTR a
    imul DWORD PTR b
    shrd eax,edx,14
    adc  eax,0
    mov  DWORD PTR c,eax
  } // asm
  return c;
} // mul64

#if 1
coord div64(long a,long b) {
long c;
  asm {
	.386
    push ebx
    mov  eax,DWORD PTR a
    mov  ebx,DWORD PTR b
    cdq
    shld edx,eax,14
    shl  eax,14
    idiv ebx
    pop  ebx
    mov  DWORD PTR c,eax
  } // asm
  return c;
} // div64
#else
coord div64(long a,long b) {
long c;
  asm {
	.386
    pushad
    mov  eax,DWORD PTR a
    mov  ebx,DWORD PTR b
    cdq
    shld edx,eax,14
    shl  eax,14
    idiv ebx
    popad
    mov  DWORD PTR c,eax
  } // asm
  return c;
} // div64

#endif

coord muldiv64(long a,long b,long c) {
long result;
  asm {
	.386
    push ecx
    mov  eax,DWORD PTR a
    mov  edx,DWORD PTR b
    imul edx
    mov  ecx,DWORD PTR c
    idiv ecx
    pop  ecx
    mov  DWORD PTR result,eax
  } // asm
  return result;
} // muldiv64

#else

coord mul64(long a,long b) {}
coord div64(long a,long b) {}
coord muldiv64(long a,long b,long c) {}

#endif


#ifdef MAKE_TABLES
static coord sintable[SIN_TABLE_SIZE];
static coord atantable[ATAN_TABLE_SIZE];
#else
static coord sintable[SIN_TABLE_SIZE] = {
0x0L,0x1921f0L,0x3243a3L,0x4b64daL,0x648557L,0x7da4dcL,0x96c32bL,0xafe006L,0xc8fb2fL,
0xe21468L,0xfb2b73L,0x1144013L,0x12d5209L,0x1466117L,0x15f6d00L,0x1787586L,0x1917a6bL,
0x1aa7b72L,0x1c3785cL,0x1dc70ecL,0x1f564e5L,0x20e5408L,0x2273e19L,0x24022daL,0x259020dL,
0x271db76L,0x28aaed6L,0x2a37bf0L,0x2bc4288L,0x2d50260L,0x2edbb3bL,0x3066cddL,0x31f1707L,
0x337b97eL,0x3505404L,0x368e65dL,0x381704dL,0x399f196L,0x3b269fcL,0x3cad943L,0x3e33f2fL,
0x3fb9b83L,0x413ee03L,0x42c3673L,0x4447498L,0x45ca835L,0x474d10fL,0x48ceeeaL,0x4a5018bL,
0x4bd08b6L,0x4d50430L,0x4ecf3beL,0x504d725L,0x51cae29L,0x5347890L,0x54c3620L,0x563e69dL,
0x57b89cdL,0x5931f77L,0x5aaa75fL,0x5c2214cL,0x5d98d03L,0x5f0ea4cL,0x60838ecL,0x61f78a9L,
0x636a94bL,0x64dca98L,0x664dc58L,0x67bde50L,0x692d049L,0x6a9b20aL,0x6c0835bL,0x6d74402L,
0x6edf3c8L,0x7049276L,0x71b1fd2L,0x7319ba6L,0x74805baL,0x75e5dd6L,0x774a3c5L,0x78ad74eL,
0x7a0f83aL,0x7b70654L,0x7cd0165L,0x7e2e936L,0x7f8bd92L,0x80e7e43L,0x8242b13L,0x839c3ccL,
0x84f483aL,0x864b826L,0x87a135dL,0x88f59aaL,0x8a48ad7L,0x8b9a6b1L,0x8cead04L,0x8e39d9cL,
0x8f87845L,0x90d3cccL,0x921eafdL,0x93682a6L,0x94b0393L,0x95f6d92L,0x973c071L,0x987fbfeL,
0x99c2006L,0x9b02c58L,0x9c420c2L,0x9d7fd14L,0x9ebc11cL,0x9ff6ca9L,0xa12ff8bL,0xa267992L,
0xa39da8dL,0xa4d224dL,0xa6050a2L,0xa73655dL,0xa86604fL,0xa994149L,0xaac081cL,0xabeb49aL,
0xad14695L,0xae3bddfL,0xaf61a4aL,0xb085baaL,0xb1a81d1L,0xb2c8c92L,0xb3e7bc2L,0xb504f33L,
0xb6206b9L,0xb73a22aL,0xb852159L,0xb96841bL,0xba7ca46L,0xbb8f3afL,0xbca002bL,0xbdaef91L,
0xbebc1b6L,0xbfc7671L,0xc0d0d99L,0xc1d8705L,0xc2de28dL,0xc3e2007L,0xc4e3f4dL,0xc5e4035L,
0xc6e2299L,0xc7de651L,0xc8d8b37L,0xc9d1124L,0xcac77f2L,0xcbbbf7aL,0xccae797L,0xcd9f023L,
0xce8d8faL,0xcf7a1f7L,0xd064af5L,0xd14d3d0L,0xd233c64L,0xd31848dL,0xd3fac29L,0xd4db314L,
0xd5b992cL,0xd695e4fL,0xd77025aL,0xd84852cL,0xd91e6a3L,0xd9f269fL,0xdac44ffL,0xdb941a2L,
0xdc61c69L,0xdd2d533L,0xddf6be2L,0xdebe056L,0xdf83270L,0xe046213L,0xe106f1fL,0xe1c5978L,
0xe282100L,0xe33c59aL,0xe3f4729L,0xe4aa590L,0xe55e0b4L,0xe60f879L,0xe6becc4L,0xe76bd7aL,
0xe816a7fL,0xe8bf3baL,0xe965910L,0xea09a68L,0xeaab7a9L,0xeb4b0b9L,0xebe8581L,0xec835e7L,
0xed1c1d4L,0xedb2931L,0xee46be5L,0xeed89dbL,0xef682fbL,0xeff5731L,0xf080665L,0xf109082L,
0xf18f574L,0xf213525L,0xf294f82L,0xf314476L,0xf3913edL,0xf40bdd5L,0xf48421bL,0xf4fa0abL,
0xf56d974L,0xf5dec64L,0xf64d969L,0xf6ba073L,0xf724171L,0xf78bc51L,0xf7f1106L,0xf853f7dL,
0xf8b47a9L,0xf91297bL,0xf96e4e4L,0xf9c79d6L,0xfa1e842L,0xfa7301dL,0xfac5158L,0xfb14be7L,
0xfb61fbeL,0xfbaccd1L,0xfbf5314L,0xfc3b27dL,0xfc7eaffL,0xfcbfc92L,0xfcfe72aL,0xfd3aabfL,
0xfd74747L,0xfdabcb8L,0xfde0b0bL,0xfe13238L,0xfe43236L,0xfe70afeL,0xfe9bc8aL,0xfec46d1L,
0xfeea9cfL,0xff0e57eL,0xff2f9d7L,0xff4e6d6L,0xff6ac76L,0xff84ab2L,0xff9c187L,0xffb10f1L,
0xffc38edL,0xffd3977L,0xffe128eL,0xffec430L,0xfff4e5aL,0xfffb10bL,0xfffec42L,0x10000000};
static coord atantable[ATAN_TABLE_SIZE] = {
0x0L,0xffffaL,0x1fffd5L,0x2fff70L,0x3ffeaaL,0x4ffd65L,0x5ffb80L,0x6ff8dbL,0x7ff556L,
0x8ff0d2L,0x9feb2fL,0xafe44dL,0xbfdc0cL,0xcfd24cL,0xdfc6efL,0xefb9d4L,0xffaaddL,
0x10f99eaL,0x11f86dbL,0x12f7193L,0x13f59f0L,0x14f3fd6L,0x15f2324L,0x16f03bdL,0x17ee182L,
0x18ebc54L,0x19e9415L,0x1ae68a7L,0x1be39ebL,0x1ce07c5L,0x1ddd217L,0x1ed98c2L,0x1fd5ba9L,
0x20d1ab0L,0x21cd5b9L,0x22c8ca8L,0x23c3f5fL,0x24bedc2L,0x25b97b6L,0x26b3d1dL,0x27addddL,
0x28a79d8L,0x29a10f5L,0x2a9a317L,0x2b93023L,0x2c8b7ffL,0x2d83a91L,0x2e7b7bdL,0x2f72f69L,
0x306a17cL,0x3160ddcL,0x325746fL,0x334d51dL,0x3442fccL,0x3538463L,0x362d2caL,0x3721aeaL,
0x3815ca9L,0x39097f1L,0x39fcca9L,0x3aefabbL,0x3be2210L,0x3cd4291L,0x3dc5c28L,0x3eb6ebfL,
0x3fa7a3fL,0x4097e94L,0x4187ba8L,0x4277165L,0x4365fb9L,0x445468dL,0x45425ceL,0x462fd68L,
0x471cd48L,0x4809559L,0x48f558aL,0x49e0dc8L,0x4acbdffL,0x4bb661eL,0x4ca0613L,0x4d89dcdL,
0x4e72d3aL,0x4f5b44aL,0x50432ebL,0x512a90dL,0x52116a1L,0x52f7b96L,0x53dd7dcL,0x54c2b66L,
0x55a7623L,0x568b805L,0x576f0feL,0x5852100L,0x59347fdL,0x5a165e7L,0x5af7ab2L,0x5bd8650L,
0x5cb88b5L,0x5d981d4L,0x5e771a2L,0x5f55812L,0x603351aL,0x61108aeL,0x61ed2c3L,0x62c934eL,
0x63a4a45L,0x647f79fL,0x6559b50L,0x6633551L,0x670c597L,0x67e4c19L,0x68bc8cfL,0x6993bb0L,
0x6a6a4b5L,0x6b403d5L,0x6c15908L,0x6cea447L,0x6dbe58bL,0x6e91ccdL,0x6f64a07L,0x7036d32L,
0x7108647L,0x71d9542L,0x72a9a1cL,0x73794d0L,0x7448559L,0x7516bb2L,0x75e47d6L,0x76b19c1L,
0x777e16eL,0x7849edaL,0x7915201L,0x79dfadfL,0x7aa9971L,0x7b72db5L,0x7c3b7a6L,0x7d03742L,
0x7dcac88L,0x7e91774L,0x7f57805L,0x801ce39L,0x80e1a0fL,0x81a5b84L,0x8269298L,0x832bf4aL,
0x83ee199L,0x84af984L,0x857070bL,0x8630a2dL,0x86f02ebL,0x87af145L,0x886d53bL,0x892aecdL,
0x89e7dfdL,0x8aa42cbL,0x8b5fd37L,0x8c1ad44L,0x8cd52f2L,0x8d8ee43L,0x8e47f39L,0x8f005d5L,
0x8fb821aL,0x906f409L,0x9125ba6L,0x91db8f1L,0x9290beeL,0x93454a0L,0x93f9309L,0x94ac72cL,
0x955f10dL,0x96110afL,0x96c2614L,0x9773142L,0x982323aL,0x98d2902L,0x998159cL,0x9a2f80eL,
0x9add05bL,0x9b89e86L,0x9c36296L,0x9ce1c8eL,0x9d8cc72L,0x9e37248L,0x9ee0e15L,0x9f89fdcL,
0xa0327a3L,0xa0da570L,0xa181947L,0xa22832dL,0xa2ce329L,0xa37393fL,0xa418575L,0xa4bc7d1L,
0xa560058L,0xa602f11L,0xa6a5401L,0xa746f2dL,0xa7e809dL,0xa888856L,0xa92865fL,0xa9c7abdL,
0xaa66578L,0xab04694L,0xaba1e1aL,0xac3ec0fL,0xacdb07aL,0xad76b62L,0xae11cceL,0xaeac4c3L,
0xaf4634aL,0xafdf868L,0xb078425L,0xb110688L,0xb1a7f98L,0xb23ef5cL,0xb2d55dbL,0xb36b31cL,
0xb400727L,0xb495202L,0xb5293b6L,0xb5bcc49L,0xb64fbc2L,0xb6e222aL,0xb773f88L,0xb8053e2L,
0xb895f42L,0xb9261adL,0xb9b5b2dL,0xba44bc7L,0xbad3385L,0xbb6126eL,0xbbee889L,0xbc7b5deL,
0xbd07a75L,0xbd93656L,0xbe1e988L,0xbea9414L,0xbf33600L,0xbfbcf56L,0xc04601cL,0xc0ce85bL,
0xc15681bL,0xc1ddf62L,0xc264e3bL,0xc2eb4abL,0xc3712bcL,0xc3f6875L,0xc47b5deL,0xc4ffaffL,
0xc5837e0L,0xc606c8aL,0xc689903L,0xc70bd54L,0xc78d986L,0xc80ed9fL,0xc88f9a8L,0xc90fdaa};
#endif

#ifdef MAKE_TABLES

void makeTables(void) {
long i;

  for (i=0; i < SIN_TABLE_SIZE; i++) {
	double val = (double)i/(double)(SIN_TABLE_SIZE-1);
	sintable[i] = sin(val*M_PI/2)*D_ONE;
  } // for

  for (i=0; i < ATAN_TABLE_SIZE; i++) {
	double val = (double)i/(double)(ATAN_TABLE_SIZE-1);
	if (val > 1.) val = 1.;
	atantable[i] = atan(val)*D_ONE;
  } /* for */

#if defined(PRINT_TABLES)

#define TABLE_FORMAT "0x%lxL"

  printf("static coord sintable[SIN_TABLE_SIZE] = {\n");
  for (i=0; i < SIN_TABLE_SIZE-1; i++) {
	printf(TABLE_FORMAT,sintable[i]);
	printf(",");
	if (i && i % 8 == 0) printf("\n");
  } // for
  printf(TABLE_FORMAT,sintable[i]);
  printf("};\n");
  printf("static coord atantable[ATAN_TABLE_SIZE] = {\n");
  for (i=0; i < ATAN_TABLE_SIZE-1; i++) {
	printf(TABLE_FORMAT,atantable[i]);
	printf(",");
	if (i && i % 8 == 0) printf("\n");
  } // for
  printf(TABLE_FORMAT,atantable[i]);
  printf("};\n");

#endif

} // makeTables

#endif

// Newton's method for square root.

ulong isqrt(ulong n) {
register ulong sqr = n;
register ulong approx,root;
#ifdef USE_FLOAT_FUNCS
  return (coord)sqrt((double)n);
#endif

  root = sqr/2;
  do {
	approx = root;
	root = (approx + sqr/approx)/2;
  } while (approx - root > 0);
  return root;

} // isqrt

coord getatan(coord sine) {
coord ta;
coord psine;
coord lo,diff;

  if (sine < 0) psine = -sine; else psine = sine;
  ta = psine >> ATAN_TABLE_SHIFT;

  if (ta >= ATAN_TABLE_SIZE-1) {
	ta = atantable[ATAN_TABLE_SIZE-1];
  } else {
	lo = atantable[ta];
	diff = atantable[ta+1] - lo;
	ta = lo + mula(diff,psine & ATAN_INTERP_MASK);
  } // if
  ta >>= FBITS; // Convert from 28 bit to 14 bit fixed point.
  if (sine < 0) return -ta; else return ta;

} // getatan

coord getatan2(coord sine,coord cosine) {
coord ps,pc,code;
#ifdef USE_FLOAT_FUNCS
  return TOFIXED(atan2(TOFLOAT(sine),TOFLOAT(cosine)));
#endif

  if (sine == 0 && cosine == 0) return 0;
  if (sine   < 0) { ps = -sine;   code =  1; } else { ps = sine; code = 0; }
  if (cosine < 0) { pc = -cosine; code |= 2; } else { pc = cosine; }

  if (ps < pc) {
	switch (code) {
	  case 0: // Cases 0 & 1 are equal.                     // +,+
	  case 1: return getatan(DIV(sine,cosine));             // -,+
	  case 2: return getatan(DIV(sine,cosine)) + RAD180DEG; // +,-
	  case 3: return getatan(DIV(sine,cosine)) - RAD180DEG; // -,-
	} // switch
  } else {
	switch (code) {
	  case 0: return  RAD090DEG - getatan(DIV(cosine,sine)); // +,+
	  case 1: return -RAD090DEG - getatan(DIV(cosine,sine)); // -,+
	  case 2: return  RAD090DEG - getatan(DIV(cosine,sine)); // +,-
	  case 3: return -RAD090DEG - getatan(DIV(cosine,sine)); // -,-
	} // switch
  } // if

  return 0; // Supress compiler "Missing return value" warning.

} // getatan2

coord lookupsin90(coord radians) {
coord ts;
coord prad;
coord lo,diff;

  if (radians < 0) prad = -DIV(radians,RAD090DEG);
  else prad = DIV(radians,RAD090DEG); // Convert to 0..1.0

  ts = prad >> SIN_TABLE_SHIFT;

  if (ts >= SIN_TABLE_SIZE-1) {
	ts = sintable[SIN_TABLE_SIZE-1];
  } else {
	lo = sintable[ts];
	diff = sintable[ts+1] - lo;
	ts = lo + mula(diff,prad & SIN_INTERP_MASK);
  } // if
  ts >>= FBITS; // Convert from 28 bit to 14 bit fixed point.
  if (radians < 0) return -ts; else return ts;

} // lookupsin90

coord getsin(coord radians) {
#ifdef USE_FLOAT_FUNCS
  return TOFIXED(sin(TOFLOAT(radians)));
#endif

  radians %= RAD360DEG;
  if (radians < 0) radians += RAD360DEG;
  if (radians <= RAD090DEG) {        //  90
	return lookupsin90(radians);
  } else if (radians <= RAD180DEG) { // 180
	return lookupsin90(RAD180DEG - radians);
  } else if (radians <= RAD270DEG) { // 270
	return -lookupsin90(radians - RAD180DEG);
  } else {                           // 360
	return -lookupsin90(RAD360DEG - radians);
  } // if

} // getsin

void sincos(sinCos * sc,coord radians) {
  sc->sin = getsin(radians);
  sc->cos = getsin(radians + RAD090DEG);
} // sincos

/* imath.c */
