;============================================================================
; SERBIH.ASM - Serial Bi-Modal Interrupt Handler
;----------------------------------------------------------------------------
; Written by Paul J. Radek
; Modified 2/23/95 jcs

; Register args symbol usage.

	.386

_TEXT16 SEGMENT BYTE PUBLIC USE16 'CODE'
	ASSUME  cs:_TEXT16,ds:_TEXT16,es:_TEXT16

	PUBLIC  sbhInt_
	PUBLIC  sbhEnd_

;=========================================================================
RXR         equ 0
TXR     equ 0
IER         equ 1
IIR         equ 2
LCR         equ 3
MCR         equ 4
LSR         equ 5
  LS_RX_DATA_RDY        equ 00000001b
  LS_OVERRUN_ERROR  equ 00000010b
  LS_PARITY_ERROR   equ 00000100b
  LS_FRAMING_ERROR  equ 00001000b
  LS_BREAK      equ 00010000b
  LS_TX_HOLD_EMPTY  equ 00100000b
  LS_TX_IDLE        equ 01000000b
MSR         equ 6
  MS_RLSD           equ 10000000b
  MS_RI                 equ     01000000b
  MS_DSR                equ 00100000b
  MS_CTS                equ 00010000b
  MS_DELTA_RLSD         equ 00001000b
  MS_DELTA_RI           equ 00000100b
  MS_DELTA_DSR          equ 00000010b
  MS_DELTA_CTS          equ 00000001b

FLG_CHECK_CTS   equ 00000001b
FLG_BLOCKED     equ 00000010b
FLG_TX_PRIME    equ 00000100b
FLG_USE_FIFO    equ 00001000b
FLG_CALLED      equ 00010000b

FIFO_WRITE_LEN  equ 16

header  struc
port    dw  0
flags   dw  0
rxhead  dw  0
rxtail  dw  0
rxcount dw  0
rxqsize dw  0
rxseg   dw  0
txhead  dw  0
txtail  dw  0
txcount dw  0
txqsize dw  0
txseg   dw  0
header  ends

sbhInt_ proc
        org 0
	jmp short @@f
hdr     header  <>
@@f:
	cli
	push    ax      ; Save register contents
	push    bx
	push    cx
	push    dx
	push    ds
	push    es
	push    si
	push    di

	push    cs
	pop ds      ; Pin DS to CS

	mov si,offset hdr   ; Load SI with offset of struct
	and si,0fffh

; if (serdata.port)
; {
	mov cx,[si].port    ; cache port address
	or  cx,cx       ; if  port is not set
	jz  @IssueEOI   ; get out

	or  [si].flags,FLG_CALLED

;   while( 1 )
;   {
@ReadIIR:
;     int_id = input( serdata.port+INT_ID ) & INT_ID_MASK;
	mov dx,IIR
	call    inp
	and al,7        ; Mask off unwanted bits

;      if ( int_id & INT_ID_PENDING )
;        break;
	test    al,1
	jnz @IssueEOI
;      switch( int_id )
;      {
	cmp al,6        ; Is it a Rec'vd character error?
	je  @ReadErr

;        case INT_ID_RX_READY: /*** Read ***/
	cmp al,4        ; Is it a Rec'vd character interrupt?
	jne @isTX       ; no
;          while (1)
;          {
@ReadChr:
;            status = input(serdata.port+LINE_STATUS);
	mov dx,LSR
	call    inp
;            if ( ! ( status & LS_RX_DATA_READY ) )
;               break;
	test    al,LS_RX_DATA_RDY
	jz  @ReadIIR
;            ch = input(serdata.port+RX_BUFFER); /* Read char and clear int */
@ReadErr:
	mov dx,RXR
	call    inp
;
	mov bx,[si].rxqsize
	cmp [si].rxcount,bx
	jae @ReadChr

;            *serdata.rx.tail++  = ch;
	mov es,[si].rxseg
	mov di,[si].rxtail
	mov es:[di],al
;            if (serdata.rx.tail == serdata.rx.end)
;               serdata.rx.tail = serdata.rx.start;
	inc di
	cmp di,bx
	jb  @saveTxTail

	xor di,di
@saveTxTail:
	mov [si].rxtail,di
	inc [si].rxcount
;          } /* while */
	jmp short @ReadChr
;          break;

;        case INT_ID_TX_EMPTY: /*** Write ***/
@isTX:
	cmp al,2
	jne @isModem
;          txHandler();

        test    [si].flags,FLG_CHECK_CTS
        jz @chkLSR ; was jne. Changed 2/23/95 jcs

	mov dx,MSR
	call    inp

	test    al,MS_CTS
	jnz @haveCTS

	or  [si].flags,FLG_BLOCKED
@JMPRD:
	jmp short @ReadIIR

@haveCTS:
	and [si].flags,NOT FLG_BLOCKED
@chkLSR:
	mov es,[si].txseg
	mov di,[si].txhead

	mov ah,FIFO_WRITE_LEN
	test    [si].flags,FLG_USE_FIFO
	jnz @useFIFO

;        call debugColors

	mov ah,1
@useFIFO:

        mov dx,LSR
	call    inp

	test    al,LS_TX_HOLD_EMPTY
        jz  @noTX

	cmp [si].txcount,0
	je  @noTX


	dec [si].txcount
	mov al,es:[di]
	inc di
	cmp di,[si].txqsize
	jb  @txByte
	xor di,di
@txByte:
	mov dx,TXR
	call    outp
	dec ah
	jnz @useFIFO
@noTX:
	mov [si].txhead,di
	jmp     short @JMPRD
;          break;

;        case INT_ID_MODEM:
@isModem:
	cmp al,0
	jne @isLine
;          status = input( serdata.port + MODEM_STATUS );
	mov dx,MSR
	call    inp
;          if ( serdata.checkCTS )
	test    [si].flags,FLG_CHECK_CTS
	jnz @ReadIIR
;          {
;            if ( status & MS_CTS )
	test    al,MS_CTS
	jnz @ReadIIR
;            { /* Clear To Send more characters */
;              if ( serdata.blocked )
;                txHandler();
;            }
;            else
;            {
;              serdata.blocked = 1;
	or  [si].flags,FLG_BLOCKED
	jnz @ReadIIR
;            }
;          }
;          break;

;        case INT_ID_LINE:
@isLine:
		;  input( serdata.port+LINE_STATUS );
	mov dx,LSR
	call    inp
	jnz @ReadIIR
;          break;
;      }
;    }
;  }
;  output(PIC_CMD,EOI);
@IssueEOI:
	mov al,20h
	out 20h,al      ; EOI CHIP #1
;}
	pop di
	pop si
	pop es
	pop ds
	pop dx
	pop cx
	pop bx
	pop ax
	iret
sbhInt_ endp

inp proc    near
	add dx,cx
	in  al,dx
	ret
inp endp

outp    proc    near
	add dx,cx
	out dx,al
	ret
outp    endp

debugColors proc near
        push ax
        push dx

        mov dx,3dah
	in  al,dx
	mov dx,3c0h
	mov al,31h
	out dx,al
	jmp $+2
	inc dx
	in  al,dx
	dec dx
	inc al
	and     al,0fh
	out dx,al

        pop dx
        pop ax

        ret
debugColors endp


sbhEnd_ label   byte
_TEXT16 ends
;=========================================================================
	end




