//
// Generic serial communication driver
// from Raven.
// Modified into a serial library tester. jcs. 2/22/95

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include "serial.h"

#define RECEIVE_BUFFER_REGISTER     0x00
#define TRANSMIT_HOLDING_REGISTER 	0x00
#define INTERRUPT_ENABLE_REGISTER	0x01
#define LINE_CONTROL_REGISTER       0x03
#define MODEM_CONTROL_REGISTER		0x04
#define LINE_STATUS_REGISTER        0x05

#define QUEUESIZE	1024

typedef enum { false, true } boolean;

static long Port;
static long Irq;
static long Com;

static long QueueHead;
static long QueueTail;
static long InputBuffer[QUEUESIZE];

static void (interrupt far *OldSerialISR)();

//=========================================================================
//
//	S_SerialISR
//
//=========================================================================

static void interrupt far S_SerialISR(void)
{
	int data;
	data = inp(Port+RECEIVE_BUFFER_REGISTER);

	InputBuffer[QueueHead] = data;
	QueueHead = (QueueHead+1)%QUEUESIZE;

	outp(0x20, 0x20);
}

//=========================================================================
//
//	S_BufferEmpty
//
//=========================================================================

static boolean S_BufferEmpty(void)
{
	return(QueueHead == QueueTail);
}

//=========================================================================
//
//	S_FlushBuffer
//
//=========================================================================

static void S_FlushBuffer(void)
{
	QueueHead = 0;
	QueueTail = 0;
}

//=========================================================================
//
//	S_ReadChar
//
//=========================================================================

static long S_ReadChar(void)
{
    long data;

	if(QueueTail == QueueHead)
	{
		return -1;
	}
	data = InputBuffer[QueueTail];
	QueueTail = (QueueTail+1)%QUEUESIZE;
    return data;
}

//=========================================================================
//
//	S_WriteChar
//
//=========================================================================

static void S_WriteChar(char data)
{
	while(!(inp(Port+LINE_STATUS_REGISTER)&0x20))
		;

    _disable();
	outp(Port+TRANSMIT_HOLDING_REGISTER, data);
    _enable();
}

//=========================================================================
//
//	S_WriteString
//
//=========================================================================

static void S_WriteString(char *data)
{
	int i;

	for(i = 0; *(data+i); i++)
	{
		S_WriteChar(*(data+i));
	}
//    S_WriteChar('\r');
}

//=========================================================================
//
//	S_FindPort
//
//=========================================================================

static void S_FindPort(short com)
{
	int i;
	static int ComPorts[] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8 };
	static int ComIrqs[] = { 4, 3, 4, 3 };

    Port = ComPorts[com];
    Irq = ComIrqs[com];
}

//=========================================================================
//
//	S_InitSerial
//
//		Initializes the tracker, by initing the com port, and
//			setting tracker mode 1
//=========================================================================

static void S_InitSerial(short com)
{
    S_FindPort(com);

// disable all uart interrupts
	outp(Port+INTERRUPT_ENABLE_REGISTER, 0 );

// set baud rate
	outp(Port+LINE_CONTROL_REGISTER, 0x83);
	outp(Port, 0x0c); // 9600 baud
	outp(Port+1, 0);

// set line control register (N81)
	outp(Port+LINE_CONTROL_REGISTER, 0x03);

	outp(Port+MODEM_CONTROL_REGISTER, 8);
	outp(Port+INTERRUPT_ENABLE_REGISTER, 1);

    OldSerialISR = _dos_getvect(Irq+8);
    _dos_setvect(Irq+8, S_SerialISR);

	outp(0x21, inp(0x21)&(~(1<<Irq)));
}

//=========================================================================
//
//	S_ShutDownSerial
//
//=========================================================================

static void S_ShutDownSerial(void)
{
	union REGS regs;

	outp(Port+INTERRUPT_ENABLE_REGISTER, 0);
	outp(Port+MODEM_CONTROL_REGISTER, 0);

	outp(0x21, inp(0x21)|(1<<Irq));

    _dos_setvect(Irq+8, OldSerialISR);
}

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

#ifdef __BORLANDC__
#pragma warn -par
#endif

long initSerial(serialData * sd,long port,long flags,long txQsize,long rxQsize) {
printf("*** Using TSERIAL tester code ***\n");
  S_InitSerial(port);
  return SER_OK;
} // initSerial

void deInitSerial(serialData * sd) {
  S_ShutDownSerial();
} // deInitSerial

long setBPS(serialData * sd,long bpsRate) {
  return 0;
} // setBPS

long writeSer(serialData * sd,char ch) {
  S_WriteChar(ch);
  return 1;
} // writeSer

// All writes are non-buffered.
void writeSerNoBuff(serialData * sd,char ch) {
  S_WriteChar(ch);
} // writeSerNoBuff

long readSer(serialData * sd) {
  return (long)S_ReadChar();
} // readSer

long rxBuffEmpty(serialData * sd) {
  return S_BufferEmpty();
} // rxBuffEmpty

long txBuffEmpty(serialData * sd) {
  return ((inp(Port+LINE_STATUS_REGISTER)&0x20) == 0);
} // txBuffEmpty

void flushRead(serialData * sd) {
  S_FlushBuffer();
} // flushRead

void flushWrite(serialData * sd) {
} // flushWrite

void flushSer(serialData * sd) {
  S_FlushBuffer();
} // flushSer

void setCheckCTS(serialData * sd,long checkCTS) {
} // setCheckCTS

void initDTR(serialData * sd) {
} // initDTR

void setDTR(serialData * sd,long state) {
} // setDTR

void setRTS(serialData * sd,long state) {
} // setRTS

long getDSR(serialData * sd) {
  return 0;
} // getDSR

long getDCD(serialData * sd) {
  return 0;
} // getDCD

long uartType(serialData * sd) {
  return 0;
} // uartType

long serOpen(serialData * sd) {
  return 1;
} // serOpen

