/* timer.c */
/* Created 12/12/94 */

#include <dos.h>
#include <time.h>
#include "timer.h"

#define USE_SYS_CLOCK

#ifdef USE_SYS_CLOCK
#define TCLOCKS_PER_SEC CLOCKS_PER_SEC
#else
#define TCLOCKS_PER_SEC (18+2) // round up from 18.2 to 20.0
#endif

#define TIMER_VECTOR 0x8

void (__interrupt __far * prevTimerISR)(void) = 0;

static long madeTimer=0;
static long timerCount=0;

static void __interrupt __far timerISR(void) {
  timerCount++;
  _chain_intr(prevTimerISR); // this is a jump (doesn't return)
} // timerISR

long timerSecs(long secs) {
  return secs*(long)TCLOCKS_PER_SEC;
} // timerSecs

long timerTSecs(long tsecs) {
  return tsecs*(long)TCLOCKS_PER_SEC/10;
} // timerTSecs

#ifdef USE_SYS_CLOCK

long timerHSecs(long hsecs) {
  return hsecs*(long)TCLOCKS_PER_SEC/100;
} // timerHSecs

long getTime(void) {
  return (long)clock();
} // getTime

#else

long getTime(void) {
  if (!madeTimer) {
    initTimer();
  } // if
  return timerCount;
} // getTime

void initTimer(void) {
  prevTimerISR = _dos_getvect(TIMER_VECTOR);
  _dos_setvect(TIMER_VECTOR,timerISR);
  madeTimer = 1;
} // initTimer

// Call deInitTimer() when not using SYS clock() and running in
// 16-bit real mode.
void deInitTimer(void) {
  if (madeTimer) _dos_setvect(TIMER_VECTOR,prevTimerISR);
  madeTimer = 0;
} // deInitTimer

#endif

long elapsedTime(long starttime,long timeout) {
  return (getTime() - starttime) >= timeout;
} // elapsedTime

void delayTime(long tics) {
long startT = getTime();
  while (!elapsedTime(startT,tics));
} // delayTime

#ifdef __WATCOMC__

float computeFrameRate(long frames,long starttime,long endtime) {
long dt;
  dt = endtime - starttime;
  if (dt <= 0) return 0.;
  return (float)frames*(float)TCLOCKS_PER_SEC/(float)dt;
} // computeFrameRate

#endif
/* timer.c */
