/* tcal.c */
/* Created 12/16/94 */
/* Ken Hawkins */

#include "imath1.h"
#include "vector1.h"
#include "tracker.h"
#include "trw.h"
#include "tcal.h"

//#define BASE_ADDR 0x4028
static long BASE_ADDR = 0x4030;

// Range (was "Gain")
#define XG_ADDR (BASE_ADDR+0x30)
#define YG_ADDR (BASE_ADDR+0x32)
#define ZG_ADDR (BASE_ADDR+0x34)
#define PG_ADDR (BASE_ADDR+0x36)
#define RG_ADDR (BASE_ADDR+0x38)

// Offsets
#define XO_ADDR (BASE_ADDR+0x20)
#define YO_ADDR (BASE_ADDR+0x22)
#define ZO_ADDR (BASE_ADDR+0x24)
#define PO_ADDR (BASE_ADDR+0x26)
#define RO_ADDR (BASE_ADDR+0x28)

// Filters
#define MF_ADDR (BASE_ADDR+0x10)
#define GF_ADDR (BASE_ADDR+0x12)

void setBaseAddr(unsigned long addr) {
  BASE_ADDR = addr;
} // setBaseAddr

unsigned long getBaseAddr(void) {
  return BASE_ADDR;
} // getBaseAddr

TrackerStatus readTrackerMiscVar(TrackerData * td,long offset,long * val) {
  return readTracker8(td,BASE_ADDR+offset,val);
} // readTrackerMiscVar

TrackerStatus readTrackerGain(TrackerData * td,TrackerAxis ta) {
  switch (ta) {
    case TA_X: return readTracker16(td,XG_ADDR,&td->magGain.x);
    case TA_Y: return readTracker16(td,YG_ADDR,&td->magGain.y);
    case TA_Z: return readTracker16(td,ZG_ADDR,&td->magGain.z);
    case TA_P: return readTracker16(td,PG_ADDR,&td->grvGain.x);
    case TA_R: return readTracker16(td,RG_ADDR,&td->grvGain.y);
  } // switch
  return TS_INVALID_PARM;
} // readTrackerGain

TrackerStatus writeTrackerGain(TrackerData * td,TrackerAxis ta) {
  switch (ta) {
    case TA_X: return writeTracker16(td,XG_ADDR,td->magGain.x);
    case TA_Y: return writeTracker16(td,YG_ADDR,td->magGain.y);
    case TA_Z: return writeTracker16(td,ZG_ADDR,td->magGain.z);
    case TA_P: return writeTracker16(td,PG_ADDR,td->grvGain.x);
    case TA_R: return writeTracker16(td,RG_ADDR,td->grvGain.y);
  } // switch
  return TS_INVALID_PARM;
} // writeTrackerGain

TrackerStatus readTrackerOffset(TrackerData * td,TrackerAxis ta) {
  switch (ta) {
    case TA_X: return readTracker16(td,XO_ADDR,&td->magOff.x);
    case TA_Y: return readTracker16(td,YO_ADDR,&td->magOff.y);
    case TA_Z: return readTracker16(td,ZO_ADDR,&td->magOff.z);
    case TA_P: return readTracker16(td,PO_ADDR,&td->grvOff.x);
    case TA_R: return readTracker16(td,RO_ADDR,&td->grvOff.y);
  } // switch
  return TS_INVALID_PARM;
} // readTrackerOffset

TrackerStatus writeTrackerOffset(TrackerData * td,TrackerAxis ta) {
  switch (ta) {
    case TA_X: return writeTracker16(td,XO_ADDR,td->magOff.x);
    case TA_Y: return writeTracker16(td,YO_ADDR,td->magOff.y);
    case TA_Z: return writeTracker16(td,ZO_ADDR,td->magOff.z);
    case TA_P: return writeTracker16(td,PO_ADDR,td->grvOff.x);
    case TA_R: return writeTracker16(td,RO_ADDR,td->grvOff.y);
  } // switch
  return TS_INVALID_PARM;
} // writeTrackerOffset

#if 0
TrackerStatus readTrackerFilter(TrackerData * td,TrackerAxis ta) {
  switch (ta) {
    case TA_X: return readTracker8(td,XF_ADDR,&td->magFilt.x);
    case TA_Y: return readTracker8(td,YF_ADDR,&td->magFilt.y);
    case TA_Z: return readTracker8(td,ZF_ADDR,&td->magFilt.z);
    case TA_P: return readTracker8(td,PF_ADDR,&td->grvFilt.x);
    case TA_R: return readTracker8(td,RF_ADDR,&td->grvFilt.y);
  } // switch
  return TS_INVALID_PARM;
} // readTrackerFilter

TrackerStatus writeTrackerFilter(TrackerData * td,TrackerAxis ta) {
  switch (ta) {
    case TA_X: return writeTracker8(td,XF_ADDR,td->magFilt.x);
    case TA_Y: return writeTracker8(td,YF_ADDR,td->magFilt.y);
    case TA_Z: return writeTracker8(td,ZF_ADDR,td->magFilt.z);
    case TA_P: return writeTracker8(td,PF_ADDR,td->grvFilt.x);
    case TA_R: return writeTracker8(td,RF_ADDR,td->grvFilt.y);
  } // switch
  return TS_INVALID_PARM;
} // writeTrackerFilter

TrackerStatus readTrackerBPS(TrackerData * td,long * bpsMode) {
  return TS_INVALID_PARM;
//  return readTracker8(td,BPSADDR,bpsMode);
} // writeTrackerBPS

TrackerStatus writeTrackerBPS(TrackerData * td,long bpsMode) {
  return TS_INVALID_PARM;
//  return writeTracker8(td,BPSADDR,bpsMode);
} // writeTrackerBPS
#endif

TrackerStatus readTrackerOffsets(TrackerData * td) {
TrackerStatus ts;
  ts = readTrackerOffset(td,TA_X);
  if (ts != TS_OK) return ts;
  ts = readTrackerOffset(td,TA_Y);
  if (ts != TS_OK) return ts;
  ts = readTrackerOffset(td,TA_Z);
  if (ts != TS_OK) return ts;
  ts = readTrackerOffset(td,TA_P);
  if (ts != TS_OK) return ts;
  ts = readTrackerOffset(td,TA_R);
  return ts;
} // readTrackerOffsets

TrackerStatus writeTrackerOffsets(TrackerData * td) {
TrackerStatus ts;
  ts = writeTrackerOffset(td,TA_X);
  if (ts != TS_OK) return ts;
  ts = writeTrackerOffset(td,TA_Y);
  if (ts != TS_OK) return ts;
  ts = writeTrackerOffset(td,TA_Z);
  if (ts != TS_OK) return ts;
  ts = writeTrackerOffset(td,TA_P);
  if (ts != TS_OK) return ts;
  ts = writeTrackerOffset(td,TA_R);
  return ts;
} // writeTrackerOffsets

TrackerStatus readTrackerGains(TrackerData * td) {
TrackerStatus ts;
  ts = readTrackerGain(td,TA_X);
  if (ts != TS_OK) return ts;
  ts = readTrackerGain(td,TA_Y);
  if (ts != TS_OK) return ts;
  ts = readTrackerGain(td,TA_Z);
  if (ts != TS_OK) return ts;
  ts = readTrackerGain(td,TA_P);
  if (ts != TS_OK) return ts;
  ts = readTrackerGain(td,TA_R);
  return ts;
} // readTrackerGains

TrackerStatus writeTrackerGains(TrackerData * td) {
TrackerStatus ts;
  ts = writeTrackerGain(td,TA_X);
  if (ts != TS_OK) return ts;
  ts = writeTrackerGain(td,TA_Y);
  if (ts != TS_OK) return ts;
  ts = writeTrackerGain(td,TA_Z);
  if (ts != TS_OK) return ts;
  ts = writeTrackerGain(td,TA_P);
  if (ts != TS_OK) return ts;
  ts = writeTrackerGain(td,TA_R);
  return ts;
} // writeTrackerGains

TrackerStatus readTrackerMagFilts(TrackerData * td) {
TrackerStatus ts;
  ts = readTracker8(td,MF_ADDR,&td->magFilts);
  return ts;
} // readTrackerMagFilts

TrackerStatus writeTrackerMagFilts(TrackerData * td) {
TrackerStatus ts;
  ts = writeTracker8(td,MF_ADDR,td->magFilts);
  return ts;
} // writeTrackerMagFilts

TrackerStatus readTrackerGrvFilts(TrackerData * td) {
TrackerStatus ts;
  ts = readTracker8(td,GF_ADDR,&td->grvFilts);
  return ts;
} // readTrackerGrvFilts

TrackerStatus writeTrackerGrvFilts(TrackerData * td) {
TrackerStatus ts;
  ts = writeTracker8(td,GF_ADDR,td->grvFilts);
  return ts;
} // writeTrackerGrvFilts

TrackerStatus readTrackerFilters(TrackerData * td) {
TrackerStatus ts;
  ts = readTrackerMagFilts(td);
  if (ts != TS_OK) return ts;
  ts = readTrackerGrvFilts(td);
  return ts;
} // readTrackerFilts

TrackerStatus writeTrackerFilters(TrackerData * td) {
TrackerStatus ts;
  ts = writeTrackerMagFilts(td);
  if (ts != TS_OK) return ts;
  ts = writeTrackerGrvFilts(td);
  return ts;
} // writeTrackerFilts

TrackerStatus readTrackerAllVars(TrackerData * td) {
TrackerStatus ts;
  ts = readTrackerOffsets(td);
  if (ts != TS_OK) return ts;
  ts = readTrackerGains(td);
  if (ts != TS_OK) return ts;
  ts = readTrackerFilters(td);
  return ts;
} // readTrackerAllVars

TrackerStatus writeTrackerAllVars(TrackerData * td) {
TrackerStatus ts;
  ts = writeTrackerOffsets(td);
  if (ts != TS_OK) return ts;
  ts = writeTrackerGains(td);
  if (ts != TS_OK) return ts;
  ts = writeTrackerFilters(td);
  return ts;
} // writeTrackerAllVars

char * getOffsetsStr(TrackerData * td) {
static char s[80];
  sprintf(s,"Offsets: x %5d y %5d z %5d p %5d r %5d",
          td->magOff.x,td->magOff.y,td->magOff.z,td->grvOff.x,td->grvOff.y);
  return s;
} // getOffsetsStr

char * getGainsStr(TrackerData * td) {
static char s[80];
  sprintf(s,"Gains:   x %5d y %5d z %5d p %5d r %5d",
          td->magGain.x,td->magGain.y,td->magGain.z,td->grvGain.x,td->grvGain.y);
  return s;
} // getGainsStr

char * getFiltersStr(TrackerData * td) {
static char s[80];
  sprintf(s,"Filters: Magnetic %5d Gravimetric %5d",td->magFilts,td->grvFilts);
  return s;
} // getFiltersStr

char * getAllVarsStr(TrackerData * td) {
static char s[256];
  sprintf(s,"%s\n%s\n%s\n",getOffsetsStr(td),getGainsStr(td),getFiltersStr(td));
  return s;
} // getAllVarsStr

/* tcal.c */
