/* tracker.c */
/* Created 12/12/94 */
/* Ken Hawkins */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "imath1.h"
#include "vector1.h"
#include "timer1.h"
#include "serial1.h"
#include "convert1.h"
#include "tracker.h"

long debugTrackerIO=0;

TrackerStatus readPacket(TrackerData * td,char * data,long size,long timeout) {
long startT;
long i;
  startT = getTime();
  i = 0;
  while (1) {
    if (!rxBuffEmpty(td->sd)) {
      *data++ = readSer(td->sd);
      if (++i == size) return TS_OK;
    } else {
      if (elapsedTime(startT,timeout)) {
        return TS_TIMEOUT;
      } // if
    } // if
  } // while
} // readPacket

void writeSerString(TrackerData * td,char * s) {
  if (debugTrackerIO) printf("Writing %s\n",s);
  while (*s) {
    writeSer(td->sd,*s);
//    delayTime(timerHSecs(6));
    s++;
  } // while
} // writeSerString

TrackerStatus resetTracker(TrackerData * td,long timeout) {
TrackerStatus ts;
long startA,startB;
long ch;
  startA = getTime();
  flushSer(td->sd);
  while (1) {
    writeSerString(td,"!\r");
    startB = getTime();
    while (1) {
      if (!rxBuffEmpty(td->sd)) {
        ch = readSer(td->sd);
        if (debugTrackerIO) printf("\nRead %c\n",ch);
        if (ch == 'O') return TS_OK;
      } // if
      if (elapsedTime(startB,timerTSecs(1))) break;
    } // while
    if (elapsedTime(startA,timeout)) return TS_TIMEOUT;
  } // while
} // resetTracker

TrackerStatus getTrackerResponse(TrackerData * td,long timeout) {
long startT;
long ch;
  startT = getTime();
  while (1) {
    if (!rxBuffEmpty(td->sd)) {
      ch = readSer(td->sd);
      if (debugTrackerIO) printf("Response: %c\n",ch);
      if (ch == 'O') return TS_OK;
      if (ch == 'E') return TS_CMD_ERROR;
    } // if
    if (elapsedTime(startT,timeout)) return TS_TIMEOUT;
  } // while
} // getTrackerResponse

TrackerStatus sendTrackerCMD(TrackerData * td,char * cmd,long timeout) {
  writeSerString(td,cmd);
  return getTrackerResponse(td,timeout);
} // sendTrackerCMD

TrackerStatus getTrackerVerStr(TrackerData * td,char * verstr,long timeout) {
TrackerStatus ts;
  writeSerString(td,"!V\r");
  ts = readPacket(td,verstr,TRACKER_VER_LEN,timeout);
  verstr[TRACKER_VER_LEN-1] = 0; // null terminate string, overwrite 'O' or 'E'.
  if (verstr[0] != 'M') {
    return TS_INVALID_VER_STR;
  } // if
  return ts;
} // getTrackerVerStr

#define HVER_OFFSET 44
#define HREV_OFFSET (HVER_OFFSET+4)
#define FVER_OFFSET (HREV_OFFSET+4)
#define FREV_OFFSET (FVER_OFFSET+4)

TrackerStatus getTrackerVerRev(TrackerData * td,
                               long * hver,long * hrev,
                               long * fver,long * frev,
                               long timeout) {
char v[TRACKER_VER_LEN];
TrackerStatus ts;
  ts = getTrackerVerStr(td,v,timeout);
  if (ts != TS_OK) return ts;
  if (!sscanf(&v[HVER_OFFSET],"%ld",hver)) *hver = 0;
  if (!sscanf(&v[HREV_OFFSET],"%ld",hrev)) *hrev = 0;
  if (!sscanf(&v[FVER_OFFSET],"%ld",fver)) *fver = 0;
  if (!sscanf(&v[FREV_OFFSET],"%ld",frev)) *frev = 0;
  return TS_OK;
} // getTrackVerVerRev

TrackerStatus initTracker(TrackerData * td,TrackerPort port,long comRate,long timeout) {
TrackerStatus ts;
long serStatus;
long serPort;

  switch (port) {
    case TP_COM1: serPort = SERIAL_PORT_COM1; break;
    case TP_COM2: serPort = SERIAL_PORT_COM2; break;
    case TP_COM3: serPort = SERIAL_PORT_COM3; break;
    case TP_COM4: serPort = SERIAL_PORT_COM4; break;
    default: return TS_NO_SERIAL;
  } // switch

  td->sd = (void *)calloc(1,sizeof(serialData));
  if (!td->sd) {
    printf("No memory for serial data.\n");
    return TS_NO_SERIAL;
  } // if

  if ((serStatus = initSerial(td->sd,serPort,USE_16550,1024L,1024L)) != SER_OK) {
    if (serStatus == SER_NO_PORT) {
      return TS_NO_SERIAL;
    } else {
      return TS_NO_MEMORY;
    } // if
  } // if

  setBPS(td->sd,comRate);
  setDTR(td->sd,1);
  setRTS(td->sd,1);
  setCheckCTS(td->sd,0);

  ts = resetTracker(td,timeout);

  return ts;

} // initTracker

TrackerStatus initTrackerSearch(TrackerData * td,TrackerPort * port,long comRate,long timeout) {
TrackerStatus ts;
TrackerPort serPort;

  for (serPort = TP_COM1; serPort <= TP_COM4; serPort++) {
    ts = initTracker(td,serPort,comRate,timeout);
    if (ts == TS_OK) {
      printf("Tracker found on COM%d.\n",serPort+1);
      *port = serPort;
      return ts;
    } else if (ts == TS_NO_MEMORY) {
      return ts;
    } // if
  } // for

  closeTracker(td);
  return TS_NO_TRACKER;

} // initTrackerSearch

void closeTracker(TrackerData * td) {
  if (td->sd) {
    deInitSerial(td->sd);
    free(td->sd);
    td->sd = 0;
  } //if
} // closeTracker

void requestTrackerData(TrackerData * td) {
  writeSer(td->sd,'S');
} // requestTrackerData

TrackerStatus readTracker(TrackerData * td,long timeout) {
unsigned char buff[32];
long i,x,y,z,p,r,yaw,pitch,roll;
unsigned char checksum;

  if (readPacket(td,buff,12,timeout) != TS_OK) {
    flushSer(td->sd);
    return TS_TIMEOUT;
  } // if

  x = (short)(buff[1] << 8) | buff[2];
  y = (short)(buff[3] << 8) | buff[4];
  z = (short)(buff[5] << 8) | buff[6];
  p = (short)(buff[7] << 8) | buff[8];
  r = (short)(buff[9] << 8) | buff[10];

  checksum = 0;
  for (i=0; i < 11; i++) checksum += buff[i];

  if (checksum != buff[11]) {
    flushSer(td->sd);
    return TS_CHECKSUM;
  } // if

  rawToEulerInt(x,y,z,p,r,&yaw,&pitch,&roll);

  td->magnetic.x = x;
  td->magnetic.y = y;
  td->magnetic.z = z;
  td->gravimetric.x = p;
  td->gravimetric.y = r;
  td->euler.x = pitch;
  td->euler.y = yaw;
  td->euler.z = roll;

  return TS_OK;

} // readTracker

TrackerStatus readTrackerEuler(TrackerData * td,long timeout) {
unsigned char buff[32];
long i,y,p,r,yaw,pitch,roll;
unsigned char checksum;

  if (readPacket(td,buff,8,timeout) != TS_OK) {
    flushSer(td->sd);
    return TS_TIMEOUT;
  } // if

  y = (short)(buff[1] << 8) | buff[2];
  p = (short)(buff[3] << 8) | buff[4];
  r = (short)(buff[5] << 8) | buff[6];

  checksum = 0;
  for (i=0; i < 7; i++) checksum += buff[i];

  if (checksum != buff[7]) return TS_CHECKSUM;

  td->euler.x = p*180; // 14 bit fixed point
  td->euler.y = y*180; // to degrees.
  td->euler.z = r*180;

  return TS_OK;

} // readTrackerEuler

char * trackerInfo(TrackerStatus ts) {
  switch (ts) {
    case TS_OK:
      return "Tracker initialized ok";
    case TS_NO_SERIAL:
      return "No serial port";
    case TS_NO_TRACKER:
      return "Tracker not present";
    case TS_TIMEOUT:
      return "Tracker is not responding";
    case TS_CHECKSUM:
      return "Tracker data is invalid";
    case TS_HEADER:
      return "Tracker header data is invalid";
    case TS_WRITEVERIFY:
      return "A verified write to tracker memory failed";
    case TS_INVALID_PARM:
      return "Invalid parameter";
    case TS_CMD_ERROR:
      return "Tracker command returned an error";
    case TS_NO_MEMORY:
      return "A memory allocation failed";
    case TS_INVALID_VER_STR:
      return "Tracker version string is invalid";
  } // switch
  return "Undefined";
} // trackerInfo

/* tracker.c */
