/* tracker.h */
/* Created 12/12/94 */
#ifndef TRACKER_H
#define TRACKER_H

#define TRACKER_VER_LEN 60

typedef enum {TP_COM1,TP_COM2,TP_COM3,TP_COM4} TrackerPort;
typedef enum {
  TS_OK,TS_NO_SERIAL,TS_NO_TRACKER,TS_TIMEOUT,
  TS_CHECKSUM,TS_HEADER,TS_WRITEVERIFY,TS_INVALID_PARM,
  TS_CMD_ERROR,TS_NO_MEMORY,TS_INVALID_VER_STR
} TrackerStatus;

typedef struct {
  void * sd;  // Serial Data
  vec3 magnetic;
  vec2 gravimetric;
  vec3 euler;
  vec3 magOff;
  vec3 magGain;
  vec2 grvOff;
  vec2 grvGain;
  long magFilts;
  long grvFilts;
} TrackerData;

extern long debugTrackerIO;

TrackerStatus initTracker(TrackerData * td,TrackerPort port,long comRate,long timeout);
TrackerStatus initTrackerSearch(TrackerData * td,TrackerPort * port,long comRate,long timeout);
void closeTracker(TrackerData * td);
TrackerStatus readPacket(TrackerData * td,char * data,long size,long timeout);
TrackerStatus readTracker(TrackerData * td,long timeout);
TrackerStatus readTrackerEuler(TrackerData * td,long timeout);
void writeSerString(TrackerData * td,char * s);
void requestTrackerData(TrackerData * td);
char * trackerInfo(TrackerStatus ts);

TrackerStatus resetTracker(TrackerData * td,long timeout);
TrackerStatus sendTrackerCMD(TrackerData * td,char * cmd,long timeout);
TrackerStatus getTrackerResponse(TrackerData * td,long timeout);

TrackerStatus getTrackerVerStr(TrackerData * td,char * str,long timeout);
TrackerStatus getTrackerVerRev(TrackerData * td,
                               long * hver,long * hrev,
                               long * fver,long * frev,
                               long timeout);

#endif

/* tracker.h */
