/* vector.c */
/* Created 12/14/94 */
/* Ken Hawkins */

#include <stdio.h>
#include "imath1.h"
#include "vector.h"

coord normalize2(vec2 * v) {
long len = DOT2(*v,*v);
  if (len > 0) {
    len = isqrt(SHL(len));
    v->x = DIV(v->x,len);
    v->y = DIV(v->y,len);
  } else {
    v->x = 0;
    v->y = ONE;
  } // if
  return len;
} // normalize2

coord normalize3(vec3 * v) {
long len = DOT3(*v,*v);
  if (len > 0) {
    len = isqrt(SHL(len));
    v->x = DIV(v->x,len);
    v->y = DIV(v->y,len);
    v->z = DIV(v->z,len);
  } else {
    v->x = v->y = 0;
    v->z = ONE;
  } // if
  return len;
} // normalize3


//*** Matrix/vector, matrix/matrix routines ***


void matrix33unitize(matrix33 * m) {
  SETVEC(m->x,ONE,0,0);
  SETVEC(m->y,0,ONE,0);
  SETVEC(m->z,0,0,ONE);
} // matrix33unitize

void rotate33(vec3 * out,matrix33 * m,vec3 * in) {
  out->x = DOT3(m->x,*in);
  out->y = DOT3(m->y,*in);
  out->z = DOT3(m->z,*in);
} // rotate33

void rotate34(vec3 * out,matrix34 * m,vec3 * in) {
  out->x = DOT3(m->m.x,*in) + m->w.x;
  out->y = DOT3(m->m.y,*in) + m->w.y;
  out->z = DOT3(m->m.z,*in) + m->w.z;
} // rotate34

void rotate34preTrans(vec3 * out,matrix34 * m,vec3 * in) {
vec3 t;
  ADDVEC(t,*in,m->w);
  out->x = DOT3(m->m.x,t);
  out->y = DOT3(m->m.y,t);
  out->z = DOT3(m->m.z,t);
} // rotate34preTrans

void rotate33rev(vec3 * out,matrix33 * m,vec3 * in) {
  out->x = MUL(m->x.x,in->x) + MUL(m->y.x,in->y) + MUL(m->z.x,in->z);
  out->y = MUL(m->x.y,in->x) + MUL(m->y.y,in->y) + MUL(m->z.y,in->z);
  out->z = MUL(m->x.z,in->x) + MUL(m->y.z,in->y) + MUL(m->z.z,in->z);
} // rotate33rev

void rotate34rev(vec3 * out,matrix34 * m,vec3 * in) {
  out->x = MUL(m->m.x.x,in->x) + MUL(m->m.y.x,in->y) + MUL(m->m.z.x,in->z) + m->w.x;
  out->y = MUL(m->m.x.y,in->x) + MUL(m->m.y.y,in->y) + MUL(m->m.z.y,in->z) + m->w.y;
  out->z = MUL(m->m.x.z,in->x) + MUL(m->m.y.z,in->y) + MUL(m->m.z.z,in->z) + m->w.z;
} // rotate34rev

void matrixMul33(matrix33 * out,matrix33 * a,matrix33 * b) {
  rotate33rev(&out->x,b,&a->x);
  rotate33rev(&out->y,b,&a->y);
  rotate33rev(&out->z,b,&a->z);
} // matrixMul33

void matrixMul34(matrix34 * out,matrix34 * a,matrix34 * b) {
  rotate33rev(&out->m.x,&b->m,&a->m.x);
  rotate33rev(&out->m.y,&b->m,&a->m.y);
  rotate33rev(&out->m.z,&b->m,&a->m.z);
  rotate33rev(&out->w,&b->m,&a->w);
  ADDVEC(out->w,b->w,a->w);
} // matrixMul34

void rotate44(vec4 * out,matrix44 * m,vec4 * in) {
  out->x = DOT4(m->x,*in);
  out->y = DOT4(m->y,*in);
  out->z = DOT4(m->z,*in);
  out->w = DOT4(m->w,*in);
} // rotate44

void rotate44rev(vec4 * out,matrix44 * m,vec4 * in) {
  out->x = MUL(m->x.x,in->x) + MUL(m->y.x,in->y) + MUL(m->z.x,in->z) + MUL(m->w.x,in->w);
  out->y = MUL(m->x.y,in->x) + MUL(m->y.y,in->y) + MUL(m->z.y,in->z) + MUL(m->w.y,in->w);
  out->z = MUL(m->x.z,in->x) + MUL(m->y.z,in->y) + MUL(m->z.z,in->z) + MUL(m->w.z,in->w);
  out->w = MUL(m->x.w,in->x) + MUL(m->y.w,in->y) + MUL(m->z.w,in->z) + MUL(m->w.w,in->w);
} // rotate44rev

void matrixMul44(matrix44 * out,matrix44 * a,matrix44 * b) {
  rotate44rev(&out->x,b,&a->x);
  rotate44rev(&out->y,b,&a->y);
  rotate44rev(&out->z,b,&a->z);
  rotate44rev(&out->w,b,&a->w);
} // matrixMul44

// *** Matrix building ***

void matrix33axis(matrix33 * m,vec3 * axis,coord angle) {
coord t,tx,ty,tz,txy,txz,tyz,sx,sy,sz;
sinCos sc;

  sincos(&sc,angle);

  t = ONE - sc.cos;

  sx = MUL(sc.sin,axis->x);
  sy = MUL(sc.sin,axis->y);
  sz = MUL(sc.sin,axis->z);

  tx = MUL(t,axis->x);
  ty = MUL(t,axis->y);
  tz = MUL(t,axis->z);

  txy = MUL(tx,axis->y);
  txz = MUL(tx,axis->z);
  tyz = MUL(ty,axis->z);

/* From Graphics Gems I, p466: */

  m->x.x = MUL(tx,axis->x) + sc.cos;  /* tx2 + c  */
  m->x.y = txy + sz;                  /* txy + sz */
  m->x.z = txz - sy;                  /* txz - sy */
  m->y.x = txy - sz;                  /* txy - sz */
  m->y.y = MUL(ty,axis->y) + sc.cos;  /* ty2 + c  */
  m->y.z = tyz + sx;                  /* tyz + sx */
  m->z.x = txz + sy;                  /* txz + sy */
  m->z.y = tyz - sx;                  /* tyz - sx */
  m->z.z = MUL(tz,axis->z) + sc.cos;  /* tz2 + c  */

} // matrix33Axis

/* vector.c */

