/* vfont.c */
/* Created 4/22/94 */
/* Copyright 1994 Ken Hawkins */
/* All Rights Reserved */

#include <stdio.h>
#include "imath1.h"
#include "vector1.h"
#include "clip1.h"

#include "vfont.h"

void drawVChar(long page,
			   vchar * vc,
               matrix34 * m,vec3 * offset,
			   vec3 * center,long color) {
vseg * vs = vc->seg;
def3 * p = vc->point;
long i;

  if (m) {
	for (i=0; i < vc->numPoints; i++,p++) {
      vec3 t;
      ADDVEC(t,p->d,*offset);
      rotate34(&p->t,m,&t);
      INCVEC(p->t,*center);
	} /* for */
    for (i=0; i < vc->numSegs; i++,vs++) {
      vec3 tl[2];
      vec2 cl[2];
      tl[0] = vs->v[0]->t;
      tl[1] = vs->v[1]->t;
      if (projClipLineStereo(cl,tl)) {
        drawLine(page,cl,color);
      } /* if */
    } /* for */
  } else {
	for (i=0; i < vc->numPoints; i++,p++) {
      ADDVEC(p->t,p->d,*offset);
      INCVEC(p->t,*center);
	} /* for */
    for (i=0; i < vc->numSegs; i++,vs++) {
      vec2 cl[2];
      cl[0] = *(vec2 *)(&vs->v[0]->t);
      cl[1] = *(vec2 *)(&vs->v[1]->t);
      if (clipLine2(cl)) {
        drawLine(page,cl,color);
      } /* if */
    } /* for */
  } /* if */

} /* drawVChar */

void drawVString(long page,
				 vfont * vf,char * s,
                 matrix34 * m,vec3 * center,
				 long alignment,long color) {
vchar * vc = vf->vc;
vec3 offset;
long i,index,incx;
  offset.y = offset.z = 0;
  incx = vf->width+vf->interCharSpace+1;
  switch (alignment) {
	case VF_ALIGN_LEFT:   offset.x = 0; break;
	case VF_ALIGN_CENTER: offset.x = -((incx*(strlen(s)-1))>>1); break;
	case VF_ALIGN_RIGHT:  offset.x = -incx*strlen(s); break;
  } /* switch */
  while (*s) {
	if ((index = *s++) < vf->numChars ) {
	  drawVChar(page,&vc[index],m,&offset,center,color);
	} /* if */
	offset.x += incx;
  } /* for */
} /* drawVString */

long allocVCharPoints(vchar * vc,long numPoints) {
  if (!(vc->point = (void *)calloc(numPoints,sizeof(*vc->point)))) return 0;
  vc->numPoints   = numPoints;
  return 1;
} /* allocVCharPoints */

long allocVCharSegs(vchar * vc,long numSegs) {
  if (!(vc->seg = (void *)calloc(numSegs,sizeof(*vc->seg)))) return 0;
  vc->numSegs   = numSegs;
  return 1;
} /* allocVCharSegs */

long allocVCharMembers(vchar * vc,long numPoints,long numSegs) {
  if (!allocVCharPoints(vc,numPoints)) return 0;
  if (!allocVCharSegs(vc,numSegs)) return 0;
  return 1;
} /* allocVCharMembers */

long allocVFontMembers(vfont * vf,long numChars) {
  if (!(vf->vc = (void *)calloc(numChars,sizeof(*vf->vc)))) return 0;
  vf->numChars = numChars;
  return 1;
} /* allocVFontMembers */

vfont * allocVFont(void) {
  return (vfont *)calloc(1,sizeof(vfont));
} /* allocVFont */

void freeVFontMembers(vfont * vf) {
vchar * vc;
long i;
  if (!vf) return;
  if (!(vc = vf->vc)) return;
  for (i=0; i < vf->numChars; i++,vc++) {
	if (vc->point) free(vc->point);
	if (vc->seg)   free(vc->seg);
  } /* for */
  free(vf->vc);
} /* freeVFontMembers */

void freeVFont(vfont * vf) {
  freeVFontMembers(vf);
  if (vf) free(vf);
} /* freeVFont */

#define READVF(d) if (fread((char *)&(d),sizeof(d),1,file) != 1) goto done
#define READVF2(d,l) if (fread((char *)(d),l,1,file) != 1) goto done

long loadVFont(char * filename,vfont * vf) {
FILE * file;
vchar * vc;
long i,j,index,status = 0;
  if (!(file = fopen(filename,"rb"))) return 0;
  READVF2(vf,sizeof(*vf) - sizeof(vc));
  if (!(allocVFontMembers(vf,vf->numChars))) goto done;
  vc = vf->vc;
  for (i=0; i < vf->numChars; i++,vc++) {
	READVF(vc->numPoints);
	if (vc->numPoints)
	  if (!(allocVCharPoints(vc,vc->numPoints))) goto done;
	for (j=0; j < vc->numPoints; j++) {
      READVF(vc->point[j].d);
//      READVF(vc->point[j].d.y);
//      READVF(vc->point[j].d.z);
    } /* for */
	READVF(vc->numSegs);
	if (vc->numSegs)
	  if (!(allocVCharSegs(vc,vc->numSegs))) goto done;
	for (j=0; j < vc->numSegs; j++) {
	  READVF(index);
	  vc->seg[j].v[0] = &vc->point[index];
//printf("seg %d index %d\n",j,index);
	  READVF(index);
	  vc->seg[j].v[1] = &vc->point[index];
//printf("seg %d index %d\n",j,index);
	} /* for */
  } /* for */
  status = 1;
done:
  fclose(file);
  return status;
} /* loadVFont */

void scaleVFont(vfont * vf,coord x,coord y,coord z) {
long i,j;
  vf->width  = MUL(vf->width,x);
  vf->height = MUL(vf->height,y);
  vf->interCharSpace = MUL(vf->interCharSpace,x);
  vf->interLineSpace = MUL(vf->interLineSpace,y);
  for (i=0; i < vf->numChars; i++) {
    for (j=0; j < vf->vc[i].numPoints; j++) {
      vf->vc[i].point[j].d.x = MUL(vf->vc[i].point[j].d.x,x);
      vf->vc[i].point[j].d.y = MUL(vf->vc[i].point[j].d.y,y);
      vf->vc[i].point[j].d.z = MUL(vf->vc[i].point[j].d.z,z);
    } // for
  } // for
} // scaleVFont

/* vfont.c */
